--[[
system_Conky_2core by arclance

own_window_title system_Conky
started     : 2012-04
created     : 2012-06-23
last updated: 2012-08-19

Changelog
2012-08-19
	clean up code to increase speed and improve formatting

2012-08-15
	redo second line to show gpu model
	
2012-08-14
	add scrolling updates

2012-08-06
	add conky_stop_xdotool() to kill xdotool if this conky is the last running conky using buttons.

2012-07-20
	fixed another crash bug in button script (click = "Command failed.")

2012-07-18
	fixed possible crash in button script it current mouse position is now returned by xdotool
	used os.time() to make sure graph tables are only updated every second (this trick works natively in lua update intervals of 1s or larger)
		fixes graph tables being populated with duplicate values when other windows are moved across the conky window or workspaces are changed.
		if this trick is used on local variables the lua script will show nothing while the window is moving across the conky window.

2012-07-17
	updated xdotool function so it does not have to be killed and restarted every update

2012-07-06
	Added fanspeed control for nvidia graphics card.
	
Credits
	Button code based on interactive conky by mrpeachy ####
	http://crunchbanglinux.org/forums/topic/18419/interactive-conky/ ####
]]

require 'cairo'
--require 'imlib2'
--require 'strict'

--#######################
scrot_mode = false
if tonumber(conky_parse('${updates}')) < 2 then --# setup persistent global variables | prevents reset when making changes to lua script
	--#### Setup Buttons ####
	conkyWindowTitle = "system_Conky" --# the title of the conky window (own_window_title) to make it easier to configure
	tempFileName = "xdo" --# the name of the temporary file to store mouse click locations in
	xdotool_window_number = nil
	abstlx = nil
	abstly = nil
	buttons = {} --# this table initially holds the values from the buttons
	buttons["toggleCommand_CPU_TIME"] = {state = 0, width = 303, height = 11}
	buttons["toggleCommand_MEM_IO"] = {state = 0, width = 299, height = 11}
	buttons["toggleTop_CPU_TIME"] = {state = 0, width = 13, height = 43, bl_x = 301}
	buttons["toggleTop_MEM_IO"] = {state = 0, width = 13, height = 43, bl_x = 301}
	buttons["toggleTop_numLines"] = {state = 0, width = 26, height = 23, bl_x = 288}
	buttons["resetInbox1"] = {state = 0, width = 38, height = 15, bl_x = 121.5, bl_y = 1051}
	buttons["resetInbox2"] = {state = 0, width = 46, height = buttons["resetInbox1"]["height"], bl_x = (buttons["resetInbox1"]["bl_x"] + buttons["resetInbox1"]["width"] + 5), bl_y = buttons["resetInbox1"]["bl_y"]}
	buttons["resetInbox3"] = {state = 0, width = 73, height = buttons["resetInbox1"]["height"], bl_x = (buttons["resetInbox2"]["bl_x"] + buttons["resetInbox2"]["width"] + 5), bl_y = buttons["resetInbox1"]["bl_y"]}
	buttons["resetInbox4"] = {state = 0, width = 94, height = buttons["resetInbox1"]["height"], bl_x = (buttons["resetInbox3"]["bl_x"] + buttons["resetInbox3"]["width"] + 5), bl_y = buttons["resetInbox1"]["bl_y"]}
	buttons["resetInbox5"] = {state = 0, width = 100, height = buttons["resetInbox1"]["height"], bl_x = (buttons["resetInbox4"]["bl_x"] + buttons["resetInbox4"]["width"] + 5), bl_y = buttons["resetInbox1"]["bl_y"]}
	--#### End Setup Buttons ####
	--#### Colors ####
	color0_R,color0_G,color0_B,color0_A = (204/255),(153/255),0,1 -- Yellow #CC9900
	color1_R,color1_G,color1_B,color1_A = (238/255),(118/255),0,1 -- Orange #EE7600
	color2_R,color2_G,color2_B,color2_A = (139/255),0,0,1 -- Red #8B0000
	color3_R,color3_G,color3_B,color3_A = (139/255),0,0,1 -- Shade Color #8B0000
	--## gradient colors
	color4_R,color4_G,color4_B = 1,(204/255),0 -- Dark Yellow #FFCC00
	color5_R,color5_G,color5_B = (204/255),0,0 -- Other Red #CC0000
	--## hddtemp colors
	color6_R,color6_G,color6_B,color6_A = (238/255),0,0,1 -- hddtemp red #EE0000
	color7_R,color7_G,color7_B,color7_A = 1,(128/255),0,1 -- hddtemp orange #FF8000
	color8_R,color8_G,color8_B,color8_A = (238/255),(238/255),0,1 -- hddtemp yellow #EEEE00
	--#### Font ####
	font = "DejaVu Sans Mono"
	bfSize = 12 --# Button Font Size
	fSize = 13 --# Normal Font Size
	--#### Setup Global Variables ####
	--## setup graph tables
	cpu_table1 = {}
	cpu_table2 = {}
	load_table = {}
	wlanup_table = {}
	ethup_table = {}
	wlandown_table = {}
	ethdown_table = {}
	--##
	bly_check = 1
	top_lines = 6
	string_Start = 1
	string_End = 10
	previousEpoch = os.time() --# initialize previous epoch (use to prevent graph tables from being screwed up when other windows are moved across the conky window)
	wlan_ext_ip = "No Data"
	gpu_temp_test = -1
	conky_mem_res_min = nil
	lua_mem_used_min = nil
	hddBarsData = nil
	updates_data = nil
	email_data = nil
	rss_data = nil
	kvirc_current = nil
	iCalendar_data = nil
end --# if tonumber(conky_parse('${updates}')) < 2 then
--#### End Setup Global Variables ###

function conky_main()
	if conky_window == nil then return end
	local cs = cairo_xlib_surface_create(conky_window.display, conky_window.drawable, conky_window.visual, conky_window.width, conky_window.height)
	local cr = cairo_create(cs)
	local updates = tonumber(conky_parse('${updates}'))
	local newEpoch = os.time()
	--#######################
	if updates == 5 then
		local cpu0fix = conky_parse("${cpu cpu0}") --# first call to cpu0 returns ''
	end --# if updates == 5 then
	if updates > 5 then
		if updates == 6 then
			cpu_table1[1] = conky_parse("${cpu cpu1}")
			cpu_table2[1] = conky_parse("${cpu cpu2}")
			load_table[1] = conky_parse("${loadavg 1}")
			wlanup_table[1] = conky_parse("${upspeedf wlan0}")
			ethup_table[1] = conky_parse("${upspeedf eth0}")
			wlandown_table[1] = conky_parse("${downspeedf wlan0}")
			ethdown_table[1] = conky_parse("${downspeedf eth0}")
		end --# if updates == 6 then
		--# draw background shading image
		--image({x=(0),y=(0),w=622,h=1082,file="/dynamic_background.png"})
		cr = draw_bg(cr)
		--#################################################################################################
		local localx,localy,localnowx,localnowy,reset_xdotool_log = clickfunction()--this line activates the clickfunction and sets the click coordinates
		--#################################################################################################
		--# start conky
		cairo_set_line_join (cr, CAIRO_LINE_JOIN_MITER) --# in case of wacky defaults
		cairo_set_antialias (cr, CAIRO_ANTIALIAS_NONE) --# don't antialias outline it is too thin and gets blurred
		local conkyWidth = conky_window.width - conky_window.text_start_x - conky_window.border_width
		cairo_select_font_face (cr, font, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL)
		cairo_set_font_size (cr, fSize)
		--# line1
		local nodename = conky_parse('${nodename}')
		local kernel = conky_parse('${kernel}')
		local workspace = conky_parse('${desktop}')
		local line1 = nodename.." #!10 64bit BPO "..kernel.." "..conky_version.." WS "..workspace
		--# draw line 1
		local extents = cairo_text_extents_t:create()
		tolua.takeownership(extents)
		cairo_text_extents(cr, "|", extents)
		local vBar_x_advance = extents.x_advance - 1
		cairo_text_extents(cr, line1, extents)
		local string_length = extents.x_advance
		local blx = (conkyWidth - string_length)/2 --# center line 1 in window
		local bly = 13
		cr = draw_shaded_text(cr, line1, blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		--# draw line 1 dividers
		cairo_text_extents(cr, nodename, extents)
		blx = blx + extents.x_advance
		cr = draw_shaded_text(cr, "|", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		cairo_text_extents(cr, " #!10 64bit BPO", extents)
		blx = blx + extents.x_advance + 1
		cr = draw_shaded_text(cr, "|", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		cairo_text_extents(cr, " "..kernel, extents)
		blx = blx + extents.x_advance
		cr = draw_shaded_text(cr, "|", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		cairo_text_extents(cr, " "..conky_version, extents)
		blx = blx + extents.x_advance - 1
		cr = draw_shaded_text(cr, "|", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		--# end line1
		--# Get Sensor Data
		local sensors = io.popen("sensors")
		local sensorData = sensors:read("*a")
		sensors:close()
		sensors = nil
		--# convert text data to list
		sensorData = sensorData:gsub(",",""):gsub("\n",'","')
		sensorData = assert(loadstring('return {"'..sensorData..'"}'))()
		--# end convert text data to list
		local core0_temp = sensorData[3]:sub(15,21)
		local core1_temp = sensorData[4]:sub(15,21)
		local cpu_freq_1 = conky_parse("${freq 1}")
		local cpu_freq_2 = conky_parse("${freq 2}")
		local gpu_freq = conky_parse("${nvidia gpufreq}")
		local gpu_mem_freq = conky_parse("${nvidia memfreq}")
		sensors = io.popen("nvidia-smi -a")
		sensorData = sensors:read("*a")
		sensors:close()
		sensors = nil
		sensorData = sensorData:gsub(",",""):gsub("\n",'","')
		sensorData = assert(loadstring('return {"'..sensorData..'"}'))()
		local gpu_model = (sensorData[9]:sub(35,52)):gsub("GeForce ","")
		local gpu_mem_used = sensorData[42]:sub(35,43):gsub(" MB","")
		local gpu_mem_total = sensorData[41]:sub(35,43):gsub(" MB","")
		local gpu_mem_percent = string.format("%.2f", (tonumber(gpu_mem_used)/tonumber(gpu_mem_total))*100)
		local gpu_temp = sensorData[83]:sub(35,40):gsub(" C","")
		local gpuFan_speed = sensorData[37]:sub(35,40):gsub(" ","")
		gpu_temp = tonumber(gpu_temp)
		if (gpu_temp_test ~= 1) and (gpu_temp >= 60) then --# control gpu fan speed based on temp
			os.execute("nvidia-settings -a [gpu:0]/GPUFanControlState=1 -a [fan:0]/GPUCurrentFanSpeed=\"12\"")
			gpu_temp_test = 1
		elseif (gpu_temp_test ~= 2) and (gpu_temp >= 45) then
			os.execute("nvidia-settings -a [gpu:0]/GPUFanControlState=1 -a [fan:0]/GPUCurrentFanSpeed=\"7\"")
			gpu_temp_test = 2
		elseif gpu_temp_test ~= 0 and (gpu_temp < 45) then
			os.execute("nvidia-settings -a [gpu:0]/GPUFanControlState=1 -a [fan:0]/GPUCurrentFanSpeed=\"4\"")
			gpu_temp_test = 0
		end --# if tonumber(gpu_temp) >= 55 then
		--# End Get Sensor Data
		--# line2
		local time_ = os.date("%r")
		cairo_text_extents(cr, time_, extents)
		local time_x_advance = extents.x_advance + 1
		local date_ = os.date("%F")
		cairo_text_extents(cr, date_, extents)
		local date_x_advance = extents.x_advance + 1
		local uptime = conky_parse('${uptime}')
		cairo_text_extents(cr, uptime, extents)
		local uptime_x_advance = extents.x_advance + 1
		local numProcesses = conky_parse('${processes}')
		cairo_text_extents(cr, numProcesses, extents)
		local numProcesses_x_advance = extents.x_advance + 1
		local numRunning = conky_parse('${running_processes}')
		cairo_text_extents(cr, numRunning, extents)
		local numRunning_x_advance = extents.x_advance + 1
		local numThreads = conky_parse('${running_threads}')
		cairo_text_extents(cr, numThreads, extents)
		local numThreads_x_advance = extents.x_advance + 1
		cairo_text_extents(cr, gpu_model, extents)
		local gpu_model_x_advance = extents.x_advance
		cairo_text_extents(cr, "Up:", extents)
		local up_x_advance = extents.x_advance
		cairo_text_extents(cr, "Proc:", extents)
		local proc_x_advance = extents.x_advance
		cairo_text_extents(cr, "Run:", extents)
		local run_x_advance = extents.x_advance
		cairo_text_extents(cr, "Thread:", extents)
		local thread_x_advance = extents.x_advance
		string_length = time_x_advance + date_x_advance + up_x_advance + uptime_x_advance + proc_x_advance + numProcesses_x_advance + run_x_advance + numRunning_x_advance + thread_x_advance + numThreads_x_advance + gpu_model_x_advance + (vBar_x_advance * 6)
		blx = (conkyWidth - string_length)/2 --# center line 1 in window
		bly = bly + 13
		cr = draw_shaded_text(cr, time_, blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = blx + time_x_advance
		cr = draw_shaded_text(cr, "|", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = blx + vBar_x_advance
		cr = draw_shaded_text(cr, date_, blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = blx + date_x_advance
		cr = draw_shaded_text(cr, "|", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = blx + vBar_x_advance
		cr = draw_shaded_text(cr, "Up:", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = blx + up_x_advance
		cr = draw_shaded_text(cr, uptime, blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = blx + uptime_x_advance
		cr = draw_shaded_text(cr, "|", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = blx + vBar_x_advance
		cr = draw_shaded_text(cr, "Proc:", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = blx + proc_x_advance
		cr = draw_shaded_text(cr, numProcesses, blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = blx + numProcesses_x_advance
		cr = draw_shaded_text(cr, "|", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = blx + vBar_x_advance
		cr = draw_shaded_text(cr, "Run:", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = blx + run_x_advance
		cr = draw_shaded_text(cr, numRunning, blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = blx + numRunning_x_advance
		cr = draw_shaded_text(cr, "|", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = blx + vBar_x_advance
		cr = draw_shaded_text(cr, "Thread:", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = blx + thread_x_advance
		cr = draw_shaded_text(cr, numThreads, blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = blx + numThreads_x_advance
		cr = draw_shaded_text(cr, "|", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = blx + vBar_x_advance
		cr = draw_shaded_text(cr, gpu_model, blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		--# end line2
		bly = bly + 5
		cr = draw_horizontal_divider(cr,1, bly, conky_window.width, bly, 1, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		--# line3
		blx = 1
		bly = bly + 13
		cr = draw_shaded_text(cr, "Overall CPU Usage:", blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		local cpu0_perc = conky_parse('${cpu cpu0}')
		blx = blx + 146
		local barlength = 440
		cr = drawBar(cr, blx, (bly), 8, barlength, 190, 1, cpu0_perc)
		blx = blx + barlength + 2
		cr = draw_shaded_text(cr, cpu0_perc.."%", blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		--# end line3
		--## CPU1 Graph ##--
		blx = 2
		bly = bly + 81
		local gvalue1 = conky_parse('${cpu cpu1}') --# graph value for this update
		local max_value = 100 --# minimum value of 
		local alpha = 0.55 --# transparency of graph
		local width = 1 --# width in pixels of the bar drawn for each table entry
		local cheight = 75 --# height of the graph box in pixels
		local table_length = 300 --# number of table entries (updates) to keep -- graph box width (in pixels) = width * table_length
		local largest_value = 0 --# holds the largest value currently displayed on on the graph
		--###################################
		if previousEpoch < newEpoch then --# only update if time has passed (lua only measures time with 1 second accuracy natively) -- prevents false updates from contaminating the table
			for i = 1, table_length do
				if cpu_table1[i+1] == nil then 
					cpu_table1[i+1] = 0 
				end --# if cpu_table1[i+1] == nil then
				cpu_table1[i] = cpu_table1[i+1]
				if i == table_length then
					cpu_table1[table_length] = tonumber(gvalue1)
				end --# if i == table_length then
				if cpu_table1[i] > largest_value then
					largest_value = cpu_table1[i]
				end --# if cpu_table1[i] > largest_value then
			end --# for i = 1, table_length do
		else
			for i = 1, table_length do
				if cpu_table1[i] > largest_value then
					largest_value = cpu_table1[i]
				end --# if cpu_table1[i] > largest_value then
			end --# for i = 1, table_length do
		end --# if previousEpoch < newEpoch then
		cr = draw_graph(cr, cpu_table1, max_value, table_length, blx, bly, alpha, width, cheight)
		cr = draw_shaded_text(cr, largest_value.."%", (blx + 2), (bly - 63), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		cairo_text_extents(cr, largest_value.."% ", extents)
		blx = blx + extents.x_advance + 1
		cr = draw_shaded_text(cr, gvalue1.."%", (blx + 2), (bly - 63), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		cairo_text_extents(cr, core0_temp, extents)
		local cpuTemp_x_advance = extents.x_advance + 1
		cairo_text_extents(cr, cpu_freq_1.." MHz", extents)
		local cpuFreq_x_advance = extents.x_advance
		local cpu_x_advance = cpuTemp_x_advance + vBar_x_advance + cpuFreq_x_advance
		blx = 300 - cpu_x_advance
		cr = draw_shaded_text(cr, core0_temp, blx, (bly - 63), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = blx + cpuTemp_x_advance
		cr = draw_shaded_text(cr, "|", blx, (bly - 63), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = blx + vBar_x_advance
		cr = draw_shaded_text(cr, cpu_freq_1.." MHz", blx, (bly - 63), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		--## End CPU1 Graph ##--
		--## CPU2 Graph ##--
		blx = 320
		local gvalue2 = conky_parse('${cpu cpu2}')
		local largest_value2 = 0
		--##### other settings same as CPU1 graph
		if previousEpoch < newEpoch then
			for i = 1, table_length do
				if cpu_table2[i+1] == nil then 
					cpu_table2[i+1] = 0
				end --# if cpu_table2[i+1] == nil then 
				cpu_table2[i] = cpu_table2[i+1]
				if i == table_length then
					cpu_table2[table_length] = tonumber(gvalue2)
				end --# if i == table_length then
				if cpu_table2[i] > largest_value2 then
					largest_value2 = cpu_table2[i]
				end --# if cpu_table2[i] > largest_value2 then
			end --# for i = 1, table_length do
		else
			for i = 1, table_length do
				if cpu_table2[i] > largest_value2 then
					largest_value2 = cpu_table2[i]
				end --# if cpu_table2[i] > largest_value2 then
			end --# for i = 1, table_length do
		end --# if previousEpoch < newEpoch then
		cr = draw_graph(cr, cpu_table2, max_value, table_length, blx, bly, alpha, width, cheight)
		cr = draw_shaded_text(cr, largest_value2.."%", (blx + 2), (bly - 63), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		cairo_text_extents(cr, largest_value2.."% ", extents)
		blx = blx + extents.x_advance + 1
		cr = draw_shaded_text(cr, gvalue2.."%", (blx + 2), (bly - 63), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		cairo_text_extents(cr, core1_temp, extents)
		cpuTemp_x_advance = extents.x_advance + 1
		cairo_text_extents(cr, cpu_freq_2.." MHz", extents)
		cpuFreq_x_advance = extents.x_advance
		cpu_x_advance = cpuTemp_x_advance + vBar_x_advance + cpuFreq_x_advance
		blx = 618 - cpu_x_advance
		cr = draw_shaded_text(cr, core1_temp, blx, (bly - 63), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = blx + cpuTemp_x_advance
		cr = draw_shaded_text(cr, "|", blx, (bly - 63), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = blx + vBar_x_advance
		cr = draw_shaded_text(cr, cpu_freq_2.." MHz", blx, (bly - 63), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		--## End CPU2 Graph ##--
		--#################################################
		--## line9
		blx = 0
		bly = bly + 13
		local line9_1 = "      "..gpu_temp.."°C".."       "..gpu_freq --.." MHz "..gpu_mem_freq.." MHz"
		cr = draw_shaded_text(cr, line9_1, blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		cairo_text_extents(cr, line9_1, extents)
		blx = blx + extents.x_advance + 3
		local line9_2 = "MHz "..gpu_mem_freq
		cr = draw_shaded_text(cr, line9_2, blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		cairo_text_extents(cr, line9_2, extents)
		blx = blx + extents.x_advance + 3
		cr = draw_shaded_text(cr, "MHz", blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = 0
		cr = draw_shaded_text(cr, "GTemp:", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		cairo_text_extents(cr, "      "..gpu_temp.."°C", extents)
		blx = blx + extents.x_advance
		cr = draw_shaded_text(cr, "|GFreq:", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		cairo_text_extents(cr, "       "..gpu_freq.."MHz", extents)
		blx = blx + extents.x_advance + 3
		cr = draw_shaded_text(cr, "/", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		cairo_text_extents(cr, "/"..gpu_mem_freq.."MHz", extents)
		blx = blx + extents.x_advance + 2
		cr = draw_shaded_text(cr, "|Fan:", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		cairo_text_extents(cr, "|Fan:", extents)
		blx = blx + extents.x_advance + 1
		cr = draw_shaded_text(cr, gpuFan_speed, blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		--## end line9
		--#################################################
		bly = bly + 5
		local bly2 = bly
		cr = draw_horizontal_divider(cr,1, bly, conky_window.width, bly, 1, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		--#################################################
		cairo_xlib_surface_set_size(cs, (conky_window.width - 1), conky_window.height)
		local topLine_Y = nil
		local color = nil
		local fill = nil
		local button_name = "toggleTop_CPU_TIME"
		buttons[button_name]["bl_y"] = bly2 + 45
		--###### calculate if click was inside box and set actions #######
		if (localx >= buttons[button_name]["bl_x"]) and (localx <= (buttons[button_name]["bl_x"] + buttons[button_name]["width"])) and (localy <= buttons[button_name]["bl_y"]) and (localy >= (buttons[button_name]["bl_y"] - buttons[button_name]["height"])) then
			if buttons[button_name]["state"] == 0 then
				buttons[button_name]["state"] = 1
			elseif buttons[button_name]["state"] == 1 then
				buttons[button_name]["state"] = 0
			end --# if buttons[button_name]["state"] == 0 then
		elseif buttons[button_name]["state"] == nil then
			buttons[button_name]["state"] = 0
		end --# if (localx >= buttons[button_name]["bl_x"]) and (localx <= (buttons[button_name]["bl_x"] + buttons[button_name]["width"])) and (localy <= buttons[button_name]["bl_y"]) and (localy >= (buttons[button_name]["bl_y"] - buttons[button_name]["height"])) then
		--##### end of calculations ###################################
		--#############################################################
		local button_name2 = "toggleCommand_CPU_TIME"
		if buttons[button_name2]["bl_y"] == nil then
			buttons[button_name2]["bl_x"] = buttons[button_name]["bl_x"] + buttons[button_name]["width"] + 2
			buttons[button_name2]["bl_y"] = buttons[button_name]["bl_y"] - buttons[button_name]["height"] + 9.5
		end --# if buttons[button_name2]["bl_y"] == nil then
		if (localx >= buttons[button_name2]["bl_x"]) and (localx <= (buttons[button_name2]["bl_x"] + buttons[button_name2]["width"])) and (localy <= buttons[button_name2]["bl_y"]) and (localy >= (buttons[button_name2]["bl_y"] - buttons[button_name2]["height"])) then
			if buttons[button_name2]["state"] ~= 1 then
				buttons[button_name2]["state"] = 1
			elseif buttons[button_name2]["state"] == 1 then
				buttons[button_name2]["state"] = 0
			end --# if buttons[button_name2]["state"] ~= 1 then
		elseif buttons[button_name2]["state"] == nil then
			buttons[button_name2]["state"] = 0
		end --# if (localx >= buttons[button_name2]["bl_x"]) and (localx <= (buttons[button_name2]["bl_x"] + buttons[button_name2]["width"])) and (localy <= buttons[button_name2]["bl_y"]) and (localy >= (buttons[button_name2]["bl_y"] - buttons[button_name2]["height"])) then
		--##############################################################
		--#### use the output of the button to set other conditions #########
		if buttons[button_name]["state"] == 1 then
			color = {color1_R,color1_G,color1_B,color1_A} --# if button is clicked on it will be drawn this color
			fill = 0
			--############################################
			cairo_select_font_face (cr, font, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL);
			cairo_set_font_size (cr, bfSize)
			cr = draw_shaded_text(cr, "T", (buttons[button_name]["bl_x"] + 2), (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + 11), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			cr = draw_shaded_text(cr, "I", (buttons[button_name]["bl_x"] + 2), (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + 21), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			cr = draw_shaded_text(cr, "M", (buttons[button_name]["bl_x"] + 2), (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + 31), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			cr = draw_shaded_text(cr, ">", (buttons[button_name]["bl_x"] + 2), (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + 41), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			cairo_select_font_face (cr, font, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL);
			cairo_set_font_size (cr, fSize)
			if buttons[button_name2]["state"] == 0 then
				cr = draw_shaded_text(cr, "Name          PID     TIME  CPU%  MEM%", (buttons[button_name]["bl_x"] + buttons[button_name]["width"] + 2), (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + 9.5), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			else
				cr = draw_shaded_text(cr, "Command", (buttons[button_name]["bl_x"] + buttons[button_name]["width"] + 2), (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + 9.5), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
				cr = draw_shaded_text(cr, "TIME", (buttons[button_name]["bl_x"] + buttons[button_name]["width"] + 273), (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + 9.5), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			end --# if buttons[button_name2]["state"] == 0 then
			topLine_Y = 22.5
			for topLines = 1 , top_lines do
				--##############################
				local top_Name = conky_parse((string.gsub("${top_time name 1}", "1", tostring(topLines))))
				local top_PID = conky_parse((string.gsub("${top_time pid 1}", "1", tostring(topLines))))
				local top_Time = conky_parse((string.gsub("${top_time time 1}", "1", tostring(topLines))))
				local BLX = buttons[button_name]["bl_x"] + buttons[button_name]["width"] + 2
				if buttons[button_name2]["state"] == 0 then
					local top_CPU = conky_parse((string.gsub("${top_time cpu 1}", "1", tostring(topLines))))
					local top_Mem = conky_parse((string.gsub("${top_time mem 1}", "1", tostring(topLines))))
					cr = draw_shaded_text(cr, top_Name, BLX, (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + topLine_Y), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
					BLX = BLX + 96
					cr = draw_shaded_text(cr, top_PID, BLX, (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + topLine_Y), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
					BLX = BLX + 41
					cr = draw_shaded_text(cr, top_Time, BLX, (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + topLine_Y), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
					BLX = BLX + 71
					cr = draw_shaded_text(cr, top_CPU, BLX, (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + topLine_Y), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
					BLX = BLX + 47
					cr = draw_shaded_text(cr, top_Mem, BLX, (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + topLine_Y), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
				else
					local top_Command = ""
					local top_Check = top_Name:gsub(" ","")
					if ((top_Check == "python") or (top_Check == "conky")) then
						top_Command = conky_parse("${pid_cmdline "..(top_PID:gsub(" ","")).."}")
					else
						top_Command = top_Name
					end --# if ((top_Check == "python") or (top_Check == "conky")) then
					local shortCommand = top_Command:sub((#top_Command - 26), #top_Command)
					if #shortCommand < 11 then
						shortCommand = top_Name
					end --# if #shortCommand < 11 then
					cr = draw_shaded_text(cr, shortCommand, BLX, (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + topLine_Y), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
					BLX = BLX + 230
					cr = draw_shaded_text(cr, top_Time, BLX, (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + topLine_Y), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
				end --# if buttons[button_name2]["state"] == 0 then
				topLine_Y = (topLine_Y + 13)
			end--for topLines=1,6 do
		else
			color = {color0_R,color0_G,color0_B,color0_A}
			fill = 0
			--############################################
			cairo_select_font_face (cr, font, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL);
			cairo_set_font_size (cr, bfSize)
			cr = draw_shaded_text(cr, "C", (buttons[button_name]["bl_x"] + 2), (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + 11), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			cr = draw_shaded_text(cr, "P", (buttons[button_name]["bl_x"] + 2), (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + 21), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			cr = draw_shaded_text(cr, "U", (buttons[button_name]["bl_x"] + 2), (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + 31), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			cr = draw_shaded_text(cr, ">", (buttons[button_name]["bl_x"] + 2), (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + 41), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			cairo_select_font_face (cr, font, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL);
			cairo_set_font_size (cr, fSize)
			if buttons[button_name2]["state"] == 0 then
				cr = draw_shaded_text(cr, "Name          PID  CPU%  MEM%     TIME", (buttons[button_name]["bl_x"] + buttons[button_name]["width"] + 2), (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + 9.5), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			else
				cr = draw_shaded_text(cr, "Command", (buttons[button_name]["bl_x"] + buttons[button_name]["width"] + 2), (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + 9.5), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
				cr = draw_shaded_text(cr, "CPU%", (buttons[button_name]["bl_x"] + buttons[button_name]["width"] + 273), (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + 9.5), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			end --# if buttons[button_name2]["state"] == 0 then
			topLine_Y = 22.5
			for topLines = 1 , top_lines do
				--#######################################
				local top_Name = conky_parse((string.gsub("${top name 1}", "1", tostring(topLines))))
				local top_PID = conky_parse((string.gsub("${top pid 1}", "1", tostring(topLines))))
				local top_CPU = conky_parse((string.gsub("${top cpu 1}", "1", tostring(topLines))))
				local BLX = buttons[button_name]["bl_x"] + buttons[button_name]["width"] + 2
				if buttons[button_name2]["state"] == 0 then
					local top_Mem = conky_parse((string.gsub("${top mem 1}", "1", tostring(topLines))))
					local top_Time = conky_parse((string.gsub("${top time 1}", "1", tostring(topLines))))
					cr = draw_shaded_text(cr, top_Name, BLX, (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + topLine_Y), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
					BLX = BLX + 96
					cr = draw_shaded_text(cr, top_PID, BLX, (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + topLine_Y), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
					BLX = BLX + 38
					cr = draw_shaded_text(cr, top_CPU, BLX, (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + topLine_Y), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
					BLX = BLX + 49
					cr = draw_shaded_text(cr, top_Mem, BLX, (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + topLine_Y), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
					BLX = BLX + 57
					cr = draw_shaded_text(cr, top_Time, BLX, (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + topLine_Y), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
				else
					local top_Command = ""
					local top_Check = top_Name:gsub(" ","")
					if ((top_Check == "python") or (top_Check == "conky")) then
						top_Command = conky_parse("${pid_cmdline "..(top_PID:gsub(" ","")).."}")
					else
						top_Command = top_Name
					end --# if ((top_Check == "python") or (top_Check == "conky")) then
					local shortCommand = top_Command:sub((#top_Command - 31), #top_Command)
					if #shortCommand < 11 then
						shortCommand = top_Name
					end --# if #shortCommand < 11 then
					cr = draw_shaded_text(cr, shortCommand, BLX, (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + topLine_Y), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
					BLX = BLX + 255
					cr = draw_shaded_text(cr, top_CPU, BLX, (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + topLine_Y), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
				end --# if buttons[button_name2]["state"] == 0 then
				--#######################################
				topLine_Y = (topLine_Y + 13)
			end--for topLines=1,6 do
		end --# if buttons[button_name]["state"] == 1 then
		--##### draw the button ####################
		cr = buttondraw(cr, buttons[button_name]["bl_x"], buttons[button_name]["bl_y"], buttons[button_name]["height"], buttons[button_name]["width"], color, fill)
		--#################################################################################################
		button_name = "toggleTop_MEM_IO"
		buttons[button_name]["bl_y"] = (buttons["toggleTop_CPU_TIME"]["bl_y"] + buttons["toggleTop_CPU_TIME"]["height"] + 4)
		--###### calculate if click was inside box and set actions #######
		if (localx >= buttons[button_name]["bl_x"]) and (localx <= (buttons[button_name]["bl_x"] + buttons[button_name]["width"])) and (localy <= buttons[button_name]["bl_y"]) and (localy >= (buttons[button_name]["bl_y"] - buttons[button_name]["height"])) then
			if buttons[button_name]["state"] == 0 then
				buttons[button_name]["state"] = 1
			elseif buttons[button_name]["state"] == 1 then
				buttons[button_name]["state"] = 0
			end --# if buttons[button_name]["state"] == 0 then
		elseif buttons[button_name]["state"] == nil then
			buttons[button_name]["state"] = 0
		end --# if (localx >= buttons[button_name]["bl_x"]) and (localx <= (buttons[button_name]["bl_x"] + buttons[button_name]["width"])) and (localy <= buttons[button_name]["bl_y"]) and (localy >= (buttons[button_name]["bl_y"] - buttons[button_name]["height"])) then
		--##### end of calculations ###################################
		--##############################################################
		button_name2 = "toggleCommand_MEM_IO"
		if buttons[button_name2]["bl_y"] == nil then
			buttons[button_name2]["bl_x"] = buttons[button_name]["bl_x"] - 300.5
			buttons[button_name2]["bl_y"] = buttons[button_name]["bl_y"] - (buttons[button_name]["height"] * 2) + 5.5
		end --# if buttons[button_name2]["bl_y"] == nil then
		if (localx >= buttons[button_name2]["bl_x"]) and (localx <= (buttons[button_name2]["bl_x"] + buttons[button_name2]["width"])) and (localy <= buttons[button_name2]["bl_y"]) and (localy >= (buttons[button_name2]["bl_y"] - buttons[button_name2]["height"])) then
			if buttons[button_name2]["state"] ~= 1 then
				buttons[button_name2]["state"] = 1
			elseif buttons[button_name2]["state"] == 1 then
				buttons[button_name2]["state"] = 0
			end --# if buttons[button_name2]["state"] ~= 1 then
		elseif buttons[button_name2]["state"] == nil then
			buttons[button_name2]["state"] = 0
		end --# if (localx >= buttons[button_name2]["bl_x"]) and (localx <= (buttons[button_name2]["bl_x"] + buttons[button_name2]["width"])) and (localy <= buttons[button_name2]["bl_y"]) and (localy >= (buttons[button_name2]["bl_y"] - buttons[button_name2]["height"])) then
		--##############################################################
		--###### use the output of the button to set other conditions #########
		if buttons[button_name]["state"] == 1 then
			color = {color1_R,color1_G,color1_B,color1_A} --#if button is clicked on it will be this color
			fill = 0
			cairo_select_font_face (cr, font, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL);
			cairo_set_font_size (cr, bfSize)
			cr = draw_shaded_text(cr, "<", (buttons[button_name]["bl_x"] + 2), (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + 11), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			cr = draw_shaded_text(cr, "I", (buttons[button_name]["bl_x"] + 2), (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + 21), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			cr = draw_shaded_text(cr, "/", (buttons[button_name]["bl_x"] + 2), (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + 31), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			cr = draw_shaded_text(cr, "O", (buttons[button_name]["bl_x"] + 2), (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + 41), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			--############################################
			cairo_select_font_face (cr, font, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL);
			cairo_set_font_size (cr, fSize)
			if buttons[button_name2]["state"] == 0 then
				cr = draw_shaded_text(cr, "Name          PID IO_R           CPU%", (buttons[button_name]["bl_x"] - 300.5), (buttons[button_name]["bl_y"] - (buttons[button_name]["height"] * 2) + 5.5), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
				cr = draw_shaded_text(cr, "IO_W", (buttons[button_name]["bl_x"] - 96.5), (buttons[button_name]["bl_y"] - (buttons[button_name]["height"] * 2) + 5.5), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
				cr = draw_shaded_text(cr, "|", (buttons[button_name]["bl_x"] - 102.5), (buttons[button_name]["bl_y"] - (buttons[button_name]["height"] * 2) + 5.5), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
				cr = draw_shaded_text(cr, "|", (buttons[button_name]["bl_x"] - 42.5), (buttons[button_name]["bl_y"] - (buttons[button_name]["height"] * 2) + 5.5), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			else
				cr = draw_shaded_text(cr, "Command", (buttons[button_name]["bl_x"] - 300.5), (buttons[button_name]["bl_y"] - (buttons[button_name]["height"] * 2) + 5.5), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
				BLX = buttons[button_name]["bl_x"] - 62
				cr = draw_shaded_text(cr, "|", BLX, (buttons[button_name]["bl_y"] - (buttons[button_name]["height"] * 2) + 5.5), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
				BLX = BLX + 6
				cr = draw_shaded_text(cr, "IO_W", BLX, (buttons[button_name]["bl_y"] - (buttons[button_name]["height"] * 2) + 5.5), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
				BLX = BLX - 63
				cr = draw_shaded_text(cr, "IO_R", BLX, (buttons[button_name]["bl_y"] - (buttons[button_name]["height"] * 2) + 5.5), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			end --# if buttons[button_name2]["state"] == 0 then
			topLine_Y = 18.5
			for topLines = 1 , top_lines do
				--################################################
				local top_Name = conky_parse((string.gsub("${top_io name 1}", "1", tostring(topLines))))
				local top_PID = conky_parse((string.gsub("${top_io pid 1}", "1", tostring(topLines))))
				local top_Read = conky_parse((string.gsub("${top_io io_read 1}", "1", tostring(topLines))))
				local top_Write = conky_parse((string.gsub("${top_io io_write 1}", "1", tostring(topLines))))
				local BLX = buttons[button_name]["bl_x"] - 300.5
				if buttons[button_name2]["state"] == 0 then
					local top_CPU = conky_parse((string.gsub("${top_io cpu 1}", "1", tostring(topLines))))
					cr = draw_shaded_text(cr, top_Name, BLX, (buttons[button_name]["bl_y"] - (buttons[button_name]["height"] * 2) + topLine_Y), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
					BLX = BLX + 97
					cr = draw_shaded_text(cr, top_PID, BLX, (buttons[button_name]["bl_y"] - (buttons[button_name]["height"] * 2) + topLine_Y), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
					BLX = BLX + 47
					cr = draw_shaded_text(cr, top_Read, BLX, (buttons[button_name]["bl_y"] - (buttons[button_name]["height"] * 2) + topLine_Y), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
					BLX = BLX + 54
					cr = draw_shaded_text(cr, "|", BLX, (buttons[button_name]["bl_y"] - (buttons[button_name]["height"] * 2) + topLine_Y), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
					BLX = BLX + 6
					cr = draw_shaded_text(cr, top_Write, BLX, (buttons[button_name]["bl_y"] - (buttons[button_name]["height"] * 2) + topLine_Y), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
					BLX = BLX + 54
					cr = draw_shaded_text(cr, "|", BLX, (buttons[button_name]["bl_y"] - (buttons[button_name]["height"] * 2) + topLine_Y), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
					BLX = BLX + 6
					cr = draw_shaded_text(cr, top_CPU:gsub(" ",""), BLX, (buttons[button_name]["bl_y"] - (buttons[button_name]["height"] * 2) + topLine_Y), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
				else
					local top_Command = ""
					local top_Check = top_Name:gsub(" ","")
					if ((top_Check == "python") or (top_Check == "conky")) then
						top_Command = conky_parse("${pid_cmdline "..(top_PID:gsub(" ","")).."}")
					else
						top_Command = top_Name
					end --# if ((top_Check == "python") or (top_Check == "conky")) then
					local shortCommand = top_Command:sub((#top_Command - 20), #top_Command)
					if #shortCommand < 11 then
						shortCommand = top_Name
					end --# if #shortCommand < 11 then
					cr = draw_shaded_text(cr, shortCommand, BLX, (buttons[button_name]["bl_y"] - (buttons[button_name]["height"] * 2) + topLine_Y), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
					BLX = buttons[button_name]["bl_x"] - 62
					cr = draw_shaded_text(cr, "|", BLX, (buttons[button_name]["bl_y"] - (buttons[button_name]["height"] * 2) + topLine_Y), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
					BLX = BLX + 6
					cr = draw_shaded_text(cr, top_Write, BLX, (buttons[button_name]["bl_y"] - (buttons[button_name]["height"] * 2) + topLine_Y), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
					BLX = BLX - 63
					cr = draw_shaded_text(cr, top_Read, BLX, (buttons[button_name]["bl_y"] - (buttons[button_name]["height"] * 2) + topLine_Y), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
				end --# if buttons[button_name2]["state"] == 0 then
				topLine_Y = (topLine_Y + 13)
			end --# for topLines = 1 , top_lines do
		else
			color = {color0_R,color0_G,color0_B,color0_A}
			fill = 0
			cairo_select_font_face (cr, font, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL);
			cairo_set_font_size (cr, bfSize)
			cr = draw_shaded_text(cr, "<", (buttons[button_name]["bl_x"] + 2), (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + 11), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			cr = draw_shaded_text(cr, "M", (buttons[button_name]["bl_x"] + 2), (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + 21), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			cr = draw_shaded_text(cr, "E", (buttons[button_name]["bl_x"] + 2), (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + 31), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			cr = draw_shaded_text(cr, "M", (buttons[button_name]["bl_x"] + 2), (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + 41), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			--############################################
			cairo_select_font_face (cr, font, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL);
			cairo_set_font_size (cr, fSize)
			if buttons[button_name2]["state"] == 0 then
				cr = draw_shaded_text(cr, "Name          PID  CPU%  MEM% MEM_RES", (buttons[button_name]["bl_x"] - 300.5), (buttons[button_name]["bl_y"] - (buttons[button_name]["height"] * 2) + 5.5), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
				cr = draw_shaded_text(cr, "|", (buttons[button_name]["bl_x"] - 67.5), (buttons[button_name]["bl_y"] - (buttons[button_name]["height"] * 2) + 5.5), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			else
				cr = draw_shaded_text(cr, "Command                  MEM% MEM_RES", (buttons[button_name]["bl_x"] - 300.5), (buttons[button_name]["bl_y"] - (buttons[button_name]["height"] * 2) + 5.5), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
				cr = draw_shaded_text(cr, "|", (buttons[button_name]["bl_x"] - 67.5), (buttons[button_name]["bl_y"] - (buttons[button_name]["height"] * 2) + 5.5), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			end --# if buttons[button_name2]["state"] == 0 then
			topLine_Y = 18.5
			for topLines = 1 , top_lines do
				--############################################
				local top_Name = conky_parse((string.gsub("${top_mem name 1}", "1", tostring(topLines))))
				local top_PID = conky_parse((string.gsub("${top_mem pid 1}", "1", tostring(topLines))))
				local top_Mem = conky_parse((string.gsub("${top_mem mem 1}", "1", tostring(topLines))))
				local top_Memres = conky_parse((string.gsub("${top_mem mem_res 1}", "1", tostring(topLines))))
				local BLX = buttons[button_name]["bl_x"] - 300.5
				if buttons[button_name2]["state"] == 0 then
					local top_CPU = conky_parse((string.gsub("${top_mem cpu 1}", "1", tostring(topLines))))
					cr = draw_shaded_text(cr, top_Name, BLX, (buttons[button_name]["bl_y"] - (buttons[button_name]["height"] * 2) + topLine_Y), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
					BLX = BLX + 97
					cr = draw_shaded_text(cr, top_PID, BLX, (buttons[button_name]["bl_y"] - (buttons[button_name]["height"] * 2) + topLine_Y), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
					BLX = BLX + 38
					cr = draw_shaded_text(cr, top_CPU, BLX, (buttons[button_name]["bl_y"] - (buttons[button_name]["height"] * 2) + topLine_Y), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
					BLX = BLX + 49
					cr = draw_shaded_text(cr, top_Mem, BLX, (buttons[button_name]["bl_y"] - (buttons[button_name]["height"] * 2) + topLine_Y), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
					BLX = BLX + 56
					cr = draw_shaded_text(cr, top_Memres, BLX, (buttons[button_name]["bl_y"] - (buttons[button_name]["height"] * 2) + topLine_Y), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
				else
					local top_Command = ""
					local top_Check = top_Name:gsub(" ","")
					if ((top_Check == "python") or (top_Check == "conky")) then
						top_Command = conky_parse("${pid_cmdline "..(top_PID:gsub(" ","")).."}")
					else
						top_Command = top_Name
					end --# if ((top_Check == "python") or (top_Check == "conky")) then
					local shortCommand = top_Command:sub((#top_Command - 22), #top_Command)
					if #shortCommand < 11 then
						shortCommand = top_Name
					end --# if #shortCommand < 11 then
					cr = draw_shaded_text(cr, shortCommand, BLX, (buttons[button_name]["bl_y"] - (buttons[button_name]["height"] * 2) + topLine_Y), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
					BLX = BLX + 184
					cr = draw_shaded_text(cr, top_Mem, BLX, (buttons[button_name]["bl_y"] - (buttons[button_name]["height"] * 2) + topLine_Y), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
					BLX = BLX + 56
					cr = draw_shaded_text(cr, top_Memres, BLX, (buttons[button_name]["bl_y"] - (buttons[button_name]["height"] * 2) + topLine_Y), {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
				end --# if buttons[button_name2]["state"] == 0 then
				cr = draw_shaded_text(cr, "|", (buttons[button_name]["bl_x"] - 67.5), (buttons[button_name]["bl_y"] - (buttons[button_name]["height"] * 2) + topLine_Y), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
				topLine_Y = (topLine_Y + 13)
			end --# for topLines = 1 , top_lines do
		end --# if buttons[button_name]["state"] == 1 then
		--##### draw the button ####################
		cr = buttondraw(cr, buttons[button_name]["bl_x"], buttons[button_name]["bl_y"], buttons[button_name]["height"], buttons[button_name]["width"], color, fill)
		cairo_xlib_surface_set_size(cs, (conky_window.width), conky_window.height)
		--#################################################################################################
		bly = bly2 + math.floor(topLine_Y) - 1
		bly2 = bly
		cr = draw_horizontal_divider(cr, 1, bly, conky_window.width, bly, 1, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		--#################################################
		button_name = "toggleTop_numLines"
		buttons[button_name]["bl_y"] = bly + 28
		--###### calculate if click was inside box and set actions #######
		if (localx >= buttons[button_name]["bl_x"]) and (localx <= (buttons[button_name]["bl_x"] + buttons[button_name]["width"])) and (localy <= buttons[button_name]["bl_y"]) and (localy >= (buttons[button_name]["bl_y"] - buttons[button_name]["height"])) then
			if buttons[button_name]["state"] == 0 then
				buttons[button_name]["state"] = 1
			elseif buttons[button_name]["state"] == 1 then
				buttons[button_name]["state"] = 0
			end --# if buttons[button_name]["state"] == 0 then
		elseif buttons[button_name]["state"] == nil then
			buttons[button_name]["state"] = 0
		end --# if (localx >= buttons[button_name]["bl_x"]) and (localx <= (buttons[button_name]["bl_x"] + buttons[button_name]["width"])) and (localy <= buttons[button_name]["bl_y"]) and (localy >= (buttons[button_name]["bl_y"] - buttons[button_name]["height"])) then
		--##### end of calculations ###################################
		--##############################################################
		--####### use the output of the button to set other conditions #########
		if buttons[button_name]["state"] == 1 then
			color = {color1_R,color1_G,color1_B,color1_A} --# if button is clicked on it will be drawn this color
			fill = 0
			--############################################
			cairo_select_font_face (cr, font, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL);
			cairo_set_font_size (cr, bfSize)
			cr = draw_shaded_text(cr, "TOP", (buttons[button_name]["bl_x"] + 2), (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + 11), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			cr = draw_shaded_text(cr, "10", (buttons[button_name]["bl_x"] + 5), (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + 21), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			top_lines = 10
		else
			color = {color0_R,color0_G,color0_B,color0_A}
			fill = 0
			--############################################
			cairo_select_font_face (cr, font, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL);
			cairo_set_font_size (cr, bfSize)
			cr = draw_shaded_text(cr, "TOP", (buttons[button_name]["bl_x"] + 2), (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + 11), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			cr = draw_shaded_text(cr, " 6 ", (buttons[button_name]["bl_x"] + 2), (buttons[button_name]["bl_y"] - buttons[button_name]["height"] + 21), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			top_lines = 6
		end --# if buttons[button_name]["state"] == 1 then
		--######### draw the button ####################
		cr = buttondraw(cr, buttons[button_name]["bl_x"], buttons[button_name]["bl_y"], buttons[button_name]["height"], buttons[button_name]["width"], color, fill)
		--##############################################
		cairo_select_font_face (cr, font, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL);
		cairo_set_font_size (cr, fSize)
		--## line17
		blx = 1
		bly = bly2 + 11
		local ram_used = conky_parse("${mem}")
		local total_ram = conky_parse("${memmax}")
		local ram_used_percent = conky_parse("${memperc}")
		local line17 = "              "..ram_used.." "..total_ram.."   "..ram_used_percent.."%"
		cr = draw_shaded_text(cr, line17, blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		cr = draw_shaded_text(cr, "RAM Usage:", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		cairo_text_extents(cr, "              "..ram_used, extents)
		blx = blx + extents.x_advance + 1
		cr = draw_shaded_text(cr, "/", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		cairo_text_extents(cr, " "..total_ram.." ", extents)
		blx = blx + extents.x_advance - 2
		cr = draw_shaded_text(cr, "-", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		--## end line17
		--## line18
		blx = 1
		bly = bly + 11
		local swap_used = conky_parse("${swap}"):gsub(" ","")
		local total_swap = conky_parse("${swapmax}")
		local swap_used_percent = conky_parse("${swapperc}")
		local line18 = "              "..swap_used.." "..total_swap.."   "..swap_used_percent.."%"
		cr = draw_shaded_text(cr, line18, blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		cr = draw_shaded_text(cr, "Swap Usage:", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		cairo_text_extents(cr, "              "..swap_used, extents)
		blx = blx + extents.x_advance
		cr = draw_shaded_text(cr, "/", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		cairo_text_extents(cr, " "..total_swap.." ", extents)
		blx = blx + extents.x_advance
		cr = draw_shaded_text(cr, "-", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		--## end line18
		--## line19
		blx = 1
		bly = bly + 11
		local line19 = "              "..gpu_mem_used.."MB "..gpu_mem_total.."MB   "..gpu_mem_percent.."%"
		cr = draw_shaded_text(cr, line19, blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		cr = draw_shaded_text(cr, "GPU MEM Usage:", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		cairo_text_extents(cr, "              "..gpu_mem_used.."MB", extents)
		blx = blx + extents.x_advance
		cr = draw_shaded_text(cr, "/", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		cairo_text_extents(cr, " "..gpu_mem_total.."MB ", extents)
		blx = blx + extents.x_advance
		cr = draw_shaded_text(cr, "-", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		--## end line19
		--## Load Graph ##--
		blx = 320
		bly = bly
		local load_avg = conky_parse('${loadavg}'):gsub(" ",'","')
		load_avg = assert(loadstring('return {"'..load_avg..'"}'))()
		local gvalue_load = load_avg[1]
		max_value = 2
		alpha = 0.5
		width = 1
		cheight = 30
		table_length = 300
		--END OF SETTINGS #########################################
		if previousEpoch < newEpoch then
			for i = 1, table_length do
				if load_table[i+1] == nil then 
					load_table[i+1] = 0 
				end --# if load_table[i+1] == nil then 
				load_table[i] = load_table[i+1]
				if i == table_length then
					load_table[table_length] = tonumber(gvalue_load)
				end --# if i == table_length then
				if load_table[i] > max_value then
					max_value = load_table[i]
				end --# if load_table[i] > max_value then
			end --# for i = 1, table_length do
		else
			for i = 1, table_length do
				if load_table[i] > max_value then
					max_value = load_table[i]
				end --# if load_table[i] > max_value then
			end --# for i = 1, table_length do
		end --# if previousEpoch < newEpoch then
		--# call graph drawing function
		cr = draw_graph(cr, load_table, max_value, table_length, blx, bly, alpha, width, cheight)
		blx = blx + 3
		local blx2 = blx
		bly = bly - 19
		cr = draw_shaded_text(cr, string.format("%.2f", max_value), blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		local load_string = load_avg[1].." "..load_avg[2].." "..load_avg[3]
		cairo_text_extents(cr, load_string, extents)
		blx = conkyWidth - extents.x_advance - 3
		cr = draw_shaded_text(cr, load_string, blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		local conky_pid_cmd = io.popen("pgrep -f -x \"conky -c /path/to/conkyrc_systemConky_2core\"")
		local conky_pid = (conky_pid_cmd:read("*a")):gsub("\n","")
		conky_pid_cmd:close()
		conky_pid_cmd = nil
		local conky_mem_res = (conky_parse("${pid_vmrss "..conky_pid.."}")):gsub(" kB","")
		conky_mem_res = tonumber(conky_mem_res)
		if conky_mem_res_min == nil then
			conky_mem_res_min = conky_mem_res
		elseif conky_mem_res < conky_mem_res_min then
			conky_mem_res_min = conky_mem_res
		end --# if conky_mem_res_min == nil then
		local lua_mem_used = collectgarbage ("count")
		if lua_mem_used_min == nil then
			lua_mem_used_min = lua_mem_used
		elseif lua_mem_used < lua_mem_used_min then
			lua_mem_used_min = lua_mem_used
		end --# if lua_mem_used_min == nil then
		local conky_mem_line = conky_pid.." "..conky_mem_res.."kB "..string.format("%.2f",lua_mem_used).." "..conky_mem_res_min.."kB "..string.format("%.2f",lua_mem_used_min)
		cr = draw_shaded_text(cr, conky_mem_line, blx2, (bly + 15), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		--## End Load Graph ##--
		--##############################################
		--## hard drive data
		bly = bly + 18
		local hddBarsFile = assert(loadfile("/path/to/hddata_conky_CB10_lua.txt"))()
		if (hddBarsFile ~= nil) and (hddBarsFile ~= "") and (type(hddBarsFile) == "table") then --# if file is empty (due to external script and lua script trying to open it at the same time) do nothing
			hddBarsData = hddBarsFile
		end --# if (hddBarsFile ~= nil) and (hddBarsFile ~= "") and (type(hddBarsFile) == "table") then
		for i = 1, #hddBarsData do
			blx = 1
			bly = bly + 13
			local hdd_used = conky_parse("${fs_used "..hddBarsData[i][2].."}")
			local hdd_size = conky_parse("${fs_size "..hddBarsData[i][2].."}")
			local hdd_free = conky_parse("${fs_free "..hddBarsData[i][2].."}")
			cr = draw_shaded_text(cr, hddBarsData[i][1], blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			cairo_text_extents(cr, hddBarsData[i][1], extents)
			blx = blx + extents.x_advance + 4
			cr = draw_shaded_text(cr, hdd_used, blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
			cairo_text_extents(cr, hdd_used, extents)
			blx = blx + extents.x_advance + 1
			cr = draw_shaded_text(cr, "/", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			cairo_text_extents(cr, "/", extents)
			blx = blx + extents.x_advance - 1
			cr = draw_shaded_text(cr, hdd_free, blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
			cairo_text_extents(cr, hdd_free, extents)
			blx = blx + extents.x_advance
			cr = draw_shaded_text(cr, "|", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			blx = blx + vBar_x_advance - 2
			cr = draw_shaded_text(cr, hdd_size, blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
			blx = 249
			cr = draw_shaded_text(cr, "|", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			blx = blx + 7
			local hdd_temp_color = nil
			if hddBarsData[i][4] ~= "    " then
				if tonumber(hddBarsData[i][4]) >= 50 then
					hdd_temp_color = {color6_R,color6_G,color6_B,color6_A}
				elseif tonumber(hddBarsData[i][4]) >= 40 then
					hdd_temp_color = {color7_R,color7_G,color7_B,color7_A}
				elseif tonumber(hddBarsData[i][4]) >= 30 then
					hdd_temp_color = {color8_R,color8_G,color8_B,color8_A}
				else
					hdd_temp_color = {color0_R,color0_G,color0_B,color0_A}
				end --# if tonumber(hddBarsData[i][4]) > 50 then
				cr = draw_shaded_text(cr, hddBarsData[i][4].."°C", blx, bly, hdd_temp_color, {color3_R,color3_G,color3_B,color3_A})
			end --# if hddBarsData[i][4] ~= "    " then
			blx = blx + 33
			cr = draw_shaded_text(cr, "|", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			blx = blx + 7
			local hdd_io_read = conky_parse("${diskio_read "..hddBarsData[i][3].."}")
			cr = draw_shaded_text(cr, hdd_io_read, blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
			blx = blx + 57
			cr = draw_shaded_text(cr, "|", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			blx = blx + 7
			local hdd_io_write = conky_parse("${diskio_write "..hddBarsData[i][3].."}")
			cr = draw_shaded_text(cr, hdd_io_write, blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
			blx = blx + 60
			local hdd_used_perc = conky_parse("${fs_used_perc "..hddBarsData[i][2].."}")
			bly = bly + 1
			local bLength = 174
			cr = drawBar(cr, blx, bly, 11, bLength, 120, 1, hdd_used_perc)
			blx = blx + 1
			bly = bly - 1
			local hdd_io_ioscheduler = conky_parse("${ioscheduler "..hddBarsData[i][3]:sub(1,-2).."}")
			cr = draw_shaded_text(cr, hdd_io_ioscheduler, blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			blx = blx + bLength
			cr = draw_shaded_text(cr, hdd_used_perc.."%", blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
			--###############
		end --# for i = 1, #hddBarsData do
		--## end hard drive data
		--##############################################
		bly = bly + 5
		cr = draw_horizontal_divider(cr,1, bly, conky_window.width, bly, 1, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		--##############################################
		--# start networking
		--## start ipline
		local wlan_int_ip
		if scrot_mode == true then
			wlan_int_ip = "Int IP"
		else
			wlan_int_ip = conky_parse("${addr wlan0}")
		end --# if scrot_mode == true then
		local wlan_conn_state = conky_parse("${if_up wlan0}1${else}0${endif}")
		if scrot_mode == true then
			wlan_ext_ip = "Ext IP"
		elseif wlan_conn_state == "1" then
			if (math.mod((updates - 6), 3600) == 0) or (wlan_ext_ip == "No Data") or (wlan_ext_ip == "Not Connected") or (wlan_ext_ip == "") or (wlan_ext_ip == "Ext IP") then
				print("'"..wlan_ext_ip.."'\n")
				local ip_check = io.popen("wget --timeout=0.75 --tries=1 --inet4-only -O - http://ip.tupeux.com | tail") --# prevent wget from hanging lua script
				wlan_ext_ip = ip_check:read("*a")
				ip_check:close()
				ip_check = nil
				print("'"..wlan_ext_ip.."'\n")
			end --# if math.mod((updates - 6), 3600) == 0 then
		else
			wlan_ext_ip = "Not Connected"
		end --# if scrot_mode == true then
		local wlan_bitrate = conky_parse("${wireless_bitrate wlan0}")
		local eth_int_ip
		if scrot_mode == true then
			eth_int_ip = "Int IP"
		else
			eth_int_ip = conky_parse("${addr eth0}")
		end --# if scrot_mode == true then
		local ipline_1 = "      "..wlan_int_ip.." "..wlan_ext_ip.." "..wlan_bitrate
		blx = 1
		bly = bly + 13
		cr = draw_shaded_text(cr, ipline_1, blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		cr = draw_shaded_text(cr, "wlan0", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		cairo_text_extents(cr, "      "..wlan_int_ip, extents)
		blx = blx + extents.x_advance + 2
		cr = draw_shaded_text(cr, "|", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		cairo_text_extents(cr, " "..wlan_ext_ip, extents)
		blx = blx + extents.x_advance
		cr = draw_shaded_text(cr, "|", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = 323
		cr = draw_shaded_text(cr, "eth0", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		local ipline_2 = "     "..eth_int_ip.." 1000 Mb/s"
		cr = draw_shaded_text(cr, ipline_2, blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		cairo_text_extents(cr, "     "..eth_int_ip, extents)
		blx = blx + extents.x_advance + 2
		cr = draw_shaded_text(cr, "|", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		--## end ipline
		--## wlan upspeed Graph ##--
		blx = 2
		bly = bly + 46
		local gvalue_wlanup = conky_parse("${upspeedf wlan0}")
		max_value = 1
		alpha = 1.0
		width = 1
		cheight = 40
		table_length = 300
		--END OF SETTINGS #########################################
		if previousEpoch < newEpoch then
			for i = 1, table_length do
				if wlanup_table[i+1] == nil then
					wlanup_table[i+1] = 0 
				end --# if wlanup_table[i+1] == nil then
				wlanup_table[i]=wlanup_table[i+1]
				if i == table_length then
					wlanup_table[table_length] = tonumber(gvalue_wlanup)
				end --# if i == table_length then
				if wlanup_table[i] > max_value then
					max_value = wlanup_table[i]
				end --# if wlanup_table[i] > max_value then
			end --# for i = 1, table_length do
		else
			for i = 1, table_length do
				if wlanup_table[i] > max_value then
					max_value = wlanup_table[i]
				end --# if wlanup_table[i] > max_value then
			end --# for i = 1, table_length do
		end --# if previousEpoch < newEpoch then
		--# call graph drawing function
		cr = draw_line_graph(cr, wlanup_table, max_value, table_length, blx, bly, alpha, width, cheight)
		blx = blx + 3
		bly = bly - 28
		cr = draw_shaded_text(cr, string.format("%.1f", max_value), blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		--## End wlan upspeed Graph
		--## eth upspeed Graph ##--
		blx = 320
		bly = bly + 28
		local gvalue_ethup = conky_parse("${upspeedf eth0}")
		max_value = 1
		alpha = 1.0
		width = 1
		cheight = 40
		table_length = 300
		--END OF SETTINGS #########################################
		if previousEpoch < newEpoch then
			for i = 1, table_length do
				if ethup_table[i+1] == nil then
					ethup_table[i+1] = 0
				end --# if ethup_table[i+1] == nil then
				ethup_table[i] = ethup_table[i+1]
				if i == table_length then
					ethup_table[table_length] = tonumber(gvalue_ethup)
				end --# if i == table_length then
				if ethup_table[i] > max_value then
					max_value = ethup_table[i]
				end --# if ethup_table[i] > max_value then
			end --# for i = 1, table_length do
		else
			for i = 1, table_length do
				if ethup_table[i] > max_value then
					max_value = ethup_table[i]
				end --# if ethup_table[i] > max_value then
			end --# for i = 1, table_length do
		end --# if previousEpoch < newEpoch then
		--# call graph drawing function
		cr = draw_line_graph(cr, ethup_table, max_value, table_length, blx, bly, alpha, width, cheight)
		blx = blx + 3
		bly = bly - 28
		cr = draw_shaded_text(cr, string.format("%.1f", max_value), blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		--## End eth upspeed Graph
		--## upspeed line
		local wlan_upspeed = conky_parse("${upspeed wlan0}"):gsub(" ","")
		local wlan_totalup = conky_parse("${totalup wlan0}")
		local wlan_link_qual = conky_parse("${wireless_link_qual_perc wlan0}")
		local eth_upspeed = conky_parse("${upspeed eth0}"):gsub(" ","")
		local eth_totalup = conky_parse("${totalup eth0}")
		blx = 2
		bly = bly + 41
		cr = draw_shaded_text(cr, "    "..wlan_upspeed.." "..wlan_totalup, blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		cr = draw_shaded_text(cr, "Up:", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		cairo_text_extents(cr, "    "..wlan_upspeed, extents)
		blx = blx + extents.x_advance + 1
		cr = draw_shaded_text(cr, "|", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = 235
		cr = draw_shaded_text(cr, "Qual:", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = blx + 42
		cr = draw_shaded_text(cr, wlan_link_qual.."%", blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = 320
		cr = draw_shaded_text(cr, "Up:", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		cr = draw_shaded_text(cr, "    "..eth_upspeed.." "..eth_totalup, blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		cairo_text_extents(cr, "    "..eth_upspeed, extents)
		blx = blx + extents.x_advance + 1
		cr = draw_shaded_text(cr, "|", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		--## end upspeed line
		--## wlan downspeed Graph ##--
		blx = 2
		bly = bly + 46
		--set value to display
		local gvalue_wlandown = conky_parse("${downspeedf wlan0}")
		max_value = 1
		alpha = 1.0
		width = 1
		cheight = 40
		table_length = 300
		--END OF SETTINGS #########################################
		if previousEpoch < newEpoch then
			for i = 1, table_length do
				if wlandown_table[i+1] == nil then
					wlandown_table[i+1] = 0
				end --# if wlandown_table[i+1] == nil then
				wlandown_table[i] = wlandown_table[i+1]
				if i == table_length then
					wlandown_table[table_length] = tonumber(gvalue_wlandown)
				end --# if i == table_length then
				if wlandown_table[i] > max_value then
					max_value = wlandown_table[i]
				end --# if wlandown_table[i] > max_value then
			end --# for i = 1, table_length do
		else
			for i = 1, table_length do
				if wlandown_table[i] > max_value then
					max_value = wlandown_table[i]
				end --# if wlandown_table[i] > max_value then
			end --# for i = 1, table_length do
		end --# if previousEpoch < newEpoch then
		--# call graph drawing function
		cr = draw_line_graph(cr, wlandown_table, max_value, table_length, blx, bly, alpha, width, cheight)
		blx = blx + 3
		bly = bly - 28
		cr = draw_shaded_text(cr, string.format("%.1f", max_value), blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		--## End wlan downspeed Graph
		--## eth downspeed Graph
		blx = 320
		bly = bly + 28
		local gvalue_ethdown = conky_parse("${downspeedf eth0}")
		max_value = 1
		alpha = 1.0
		width = 1
		cheight = 40
		table_length = 300
		--END OF SETTINGS #########################################
		if previousEpoch < newEpoch then
			for i = 1, table_length do
				if ethdown_table[i+1] == nil then
					ethdown_table[i+1] = 0
				end --# if ethdown_table[i+1] == nil then
				ethdown_table[i] = ethdown_table[i+1]
				if i == table_length then
					ethdown_table[table_length] = tonumber(gvalue_ethdown)
				end --# if i == table_length then
				if ethdown_table[i] > max_value then
					max_value = ethdown_table[i]
				end --# if ethdown_table[i] > max_value then
			end --# for i = 1, table_length do
		else
			for i = 1, table_length do
				if ethdown_table[i] > max_value then
					max_value = ethdown_table[i]
				end --# if ethdown_table[i] > max_value then
			end --# for i = 1, table_length do
		end --# if previousEpoch < newEpoch then
		--# call graph drawing function
		cr = draw_line_graph(cr, ethdown_table, max_value, table_length, blx, bly, alpha, width, cheight)
		blx = blx + 3
		bly = bly - 28
		cr = draw_shaded_text(cr, string.format("%.1f", max_value), blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		--## End eth downspeed Graph
		--## start downline
		local wlan_downspeed = conky_parse("${downspeed wlan0}"):gsub(" ","")
		local wlan_totaldown = conky_parse("${totaldown wlan0}")
		local eth_downspeed = conky_parse("${downspeed eth0}"):gsub(" ","")
		local eth_totaldown = conky_parse("${totaldown eth0}")
		blx = 2
		bly = bly + 41
		cr = draw_shaded_text(cr, "      "..wlan_downspeed.." "..wlan_totaldown, blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		cr = draw_shaded_text(cr, "Down:", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		cairo_text_extents(cr, "      "..wlan_downspeed, extents)
		blx = blx + extents.x_advance + 1
		cr = draw_shaded_text(cr, "|", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = 320
		cr = draw_shaded_text(cr, "Down:", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		cr = draw_shaded_text(cr, "      "..eth_downspeed.." "..eth_totaldown, blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		cairo_text_extents(cr, "      "..eth_downspeed, extents)
		blx = blx + extents.x_advance + 1
		cr = draw_shaded_text(cr, "|", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		--## end downline
		--# end networking
		--################################################
		bly = bly + 5
		cr = draw_horizontal_divider(cr,1, bly, conky_window.width, bly, 1, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		--#################################################
		--## apcupd output
		sensors = io.popen("apcaccess status")
		sensorData = sensors:read("*a")
		sensors:close()
		sensors = nil
		local a,b = string.find(sensorData, "LINEV    : %d*.%d*%s%a*\n")
		local apc_ups_linev = sensorData:sub((a + 11),(b - 1))
		a,b = string.find(sensorData, "STATUS   : %a*%s\n")
		local apc_ups_status = sensorData:sub((a + 11),(b - 2))
		a,b = string.find(sensorData, "BCHARGE  : %d*.%d*")
		local apc_ups_charge_percent = sensorData:sub((a + 11),(b))
		a,b = string.find(sensorData, "TIMELEFT : .-%d*.%d*%s%a-\n")
		local apc_ups_time_left = sensorData:sub((a + 11),(b - 1)):gsub(" ","",1)
		a,b = string.find(sensorData, "LOADPCT  : %s*%d*.%d*")
		local apc_ups_load_perc = sensorData:sub((a + 11),(b)):gsub(" ","")
		local apc_ups_load_watts = (tonumber(apc_ups_load_perc)/100)*450 --# for a 450 watt ups
		a,b = string.find(sensorData, "LASTXFER : .-\n")
		local apc_ups_event_reason = sensorData:sub((a + 11),(b - 1))
		blx = 2
		bly = bly + 12
		cr = draw_shaded_text(cr, "Power Supplied By:", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = 153
		cr = draw_shaded_text(cr, apc_ups_status, blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = 305
		cr = draw_shaded_text(cr, "Battery Capacity:", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = 455
		cr = draw_shaded_text(cr, apc_ups_charge_percent.."%", blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = 2
		bly = bly + 13
		cr = draw_shaded_text(cr, "Utility Voltage:", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = 153
		cr = draw_shaded_text(cr, apc_ups_linev, blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = 305
		cr = draw_shaded_text(cr, "Remaining Runtime:", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = 455
		cr = draw_shaded_text(cr, apc_ups_time_left, blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = 2
		bly = bly + 13
		cr = draw_shaded_text(cr, "Last Power Event:", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = 153
		cr = draw_shaded_text(cr, apc_ups_event_reason, blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = 409
		cr = draw_shaded_text(cr, "Load:", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = 455
		cr = draw_shaded_text(cr, string.format("%.1f", apc_ups_load_watts).." Watts("..apc_ups_load_perc.."%)", blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		--# end apcupsd output
		--############################################
		bly = bly + 5
		cr = draw_horizontal_divider(cr,1, bly, conky_window.width, bly, 1, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		--############################################
		--#################################################
		--####### Update Check ######
		local updates_function = assert(loadfile("/path/to/updateCheck_log_CB10_lua.txt"))
		updates_function = updates_function()
		if (updates_function ~= nil) and (updates_function ~= "") and (type(updates_function) == "table") then --# if file is empty (due to external script and lua script trying to open it at the same time) do nothing
			updates_data = updates_function
		end --# if if (updates_function ~= nil) and (updates_function ~= "") and (type(updates_function) == "table") then
		blx = 2
		bly = bly + 12
		local string_x_advance = nil
		for i = 1, #updates_data[1] do
			if i == (#updates_data[1] - 2) then
				color = {color1_R,color1_G,color1_B,color1_A}
				updates_data[1][i] = updates_data[1][i].."|"
			elseif i == (#updates_data[1] - 1) then
				cairo_text_extents(cr, updates_data[1][i], extents)
				string_x_advance = extents.x_advance
				blx = blx + string_x_advance
				color = {color1_R,color1_G,color1_B,color1_A}
				cr = draw_shaded_text(cr, "|", blx, bly, color, {color3_R,color3_G,color3_B,color3_A})
				blx = blx - string_x_advance - 1
				color = {color0_R,color0_G,color0_B,color0_A}
			elseif i == #updates_data[1] then
				cairo_text_extents(cr, "|", extents)
				string_x_advance = extents.x_advance
				blx = blx + string_x_advance - 1
				color = {color0_R,color0_G,color0_B,color0_A}
			else
				if type(updates_data[1][i]) == "string" then
					color = {color1_R,color1_G,color1_B,color1_A}
				else
					color = {color0_R,color0_G,color0_B,color0_A}
				end --# if type(updates_data[1][i]) == "string" then
				updates_data[1][i] = updates_data[1][i].." "
			end --# if i == #updates_data[1] then
			cr = draw_shaded_text(cr, updates_data[1][i], blx, bly, color, {color3_R,color3_G,color3_B,color3_A})
			cairo_text_extents(cr, updates_data[1][i], extents)
			blx = blx + extents.x_advance
		end --# for i = 1, #updates_data[1] do
		--### show which packages will be updated ###
		if updates_data[2] ~= "" then
			blx = 2
			bly = bly + 12
			local scrollAdvance = 5 --# number of characters to advance scroll string by
			local scrollDisplay_Length = 77 --# length of scrolling text in characters
			local displayString = ""
			if #updates_data[2] > scrollDisplay_Length then
				string_End = string_End + scrollAdvance
				if string_End > scrollDisplay_Length then
					string_Start = string_End - (scrollDisplay_Length - 1)
					if string_Start > #updates_data[2] then
						string_Start = 1
						string_End = 1 + scrollAdvance*2
						displayString = updates_data[2]:sub(string_Start, string_End)
						cairo_text_extents(cr, " ", extents)
						blx = blx + (extents.x_advance * (scrollDisplay_Length - #displayString))
					else
						displayString = updates_data[2]:sub(string_Start, string_End)
					end --# if string_Start > #updates_data[3] then
				else
					displayString = updates_data[2]:sub(string_Start, string_End)
					if #displayString < scrollDisplay_Length then
						cairo_text_extents(cr, " ", extents)
						blx = blx + (extents.x_advance * (scrollDisplay_Length - #displayString))
					end --# if #displayString < scrollDisplay_Length then
				end --# if string_End > scrollDisplay_Length then
				local displaySplit = assert(loadstring(("return {'"..(displayString:gsub(" ","','")).."'}")))()
				for i = 1, #displaySplit do
					cr = draw_shaded_text(cr, displaySplit[i], blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
					cairo_text_extents(cr, displaySplit[i], extents)
					blx = blx + extents.x_advance + 1
					if i ~= #displaySplit then
						cr = draw_shaded_text(cr, "|", blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
						blx = blx + vBar_x_advance
					end --# if j ~= #updates_data[2][i] then
				end --# for j = 1, #updates_data[2][i] do
			else
				local displaySplit = assert(loadstring(("return {'"..(updates_data[2]:gsub(" ","','")).."'}")))()
				for i = 1, #displaySplit do
					cr = draw_shaded_text(cr, displaySplit[i], blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
					if i ~= #displaySplit then
						cairo_text_extents(cr, displaySplit[i], extents)
						blx = blx + extents.x_advance + 1
						cr = draw_shaded_text(cr, "|", blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
						blx = blx + 6
					end --# if i == #displaySplit then
				end --# for i = 1, #displaySplit do
			end --# if #updates_data[2] > scrollDisplay_Length then
		end --# if #updates_data[2] > 1 then
		--####### Update Check ######
		--#################################################
		bly = bly + 5
		cr = draw_horizontal_divider(cr, 1, bly, conky_window.width, bly, 1, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		--#################################################
		blx = 2
		bly = bly + 12
		local bgLogFile = io.open("/path/to/currentBG_log_lua.txt","r")
		local terminalAlpha = bgLogFile:read("*l")
		cr = draw_shaded_text(cr, terminalAlpha, blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		cairo_text_extents(cr, terminalAlpha, extents)
		blx = blx + extents.x_advance + 1
		cr = draw_shaded_text(cr, "|", blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = blx + 6
		local conkyAlpha = bgLogFile:read("*l")
		cr = draw_shaded_text(cr, conkyAlpha, blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		cairo_text_extents(cr, conkyAlpha, extents)
		blx = blx + extents.x_advance + 1
		cr = draw_shaded_text(cr, "|", blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		blx = blx + 6
		local bgName = bgLogFile:read("*l")
		cr = draw_shaded_text(cr, bgName, blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		bgLogFile:close()
		bgLogFile = nil
		--#################################################
		bly = bly + 5
		cr = draw_horizontal_divider(cr, 1, bly, conky_window.width, bly, 1, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		--#################################################
		--#### Evolution Email Data ####
		local email_function = assert(loadfile("/path/to/evolution_conky_CB10_lua_log.txt"))
		email_function = email_function()
		if (email_function ~= nil) and (email_function ~= "") and (type(email_function) == "table") then --# if file is empty (due to external script and lua script trying to open it at the same time) do nothing
			email_data = email_function
		end --# if if (updates_function ~= nil) and (updates_function ~= "") and (type(updates_function) == "table") then
		if scrot_mode == false then
			for i = 1, #email_data do
				if email_data[i][1] == 0 then
					blx = 2
					bly = bly + 12
					cr = draw_shaded_text(cr, "No New Messages In", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
					cairo_text_extents(cr, "No New Messages In ", extents)
					blx = blx + extents.x_advance + 1
					cr = draw_shaded_text(cr, email_data[i][2], blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
				elseif email_data[i][3] ~= "" then
					blx = 2
					bly = bly + 12
					cr = draw_shaded_text(cr, "(", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
					cairo_text_extents(cr, "(", extents)
					blx = blx + extents.x_advance
					cr = draw_shaded_text(cr, email_data[i][1], blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
					cairo_text_extents(cr, email_data[i][1], extents)
					blx = blx + extents.x_advance
					cr = draw_shaded_text(cr, ") New Messages In", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
					cairo_text_extents(cr, ") New Messages In ", extents)
					blx = blx + extents.x_advance + 1
					cr = draw_shaded_text(cr, email_data[i][2], blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
					cairo_text_extents(cr, email_data[i][2], extents)
					blx = blx + extents.x_advance + 1
					cr = draw_shaded_text(cr, " From", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
					cairo_text_extents(cr, " From ", extents)
					blx = blx + extents.x_advance + 1
					cr = draw_shaded_text(cr, email_data[i][3], blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
					if email_data[i][4] ~= "" then
						blx = 2
						bly = bly + 12
						cr = draw_shaded_text(cr, '"', blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
						cairo_text_extents(cr, '"', extents)
						blx = blx + extents.x_advance
						cr = draw_shaded_text(cr, email_data[i][4], blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
						cairo_text_extents(cr, email_data[i][4], extents)
						blx = blx + extents.x_advance
						cr = draw_shaded_text(cr, '"', blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
					end --# if email_data[i][4] ~= "" then
				end --# if email_data[1][i] == 0 then
			end --# for i == 1, #email_data do
		else
			blx = 2
			bly = bly + 12
			cr = draw_shaded_text(cr, "No New Messages In", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			cairo_text_extents(cr, "No New Messages In ", extents)
			blx = blx + extents.x_advance + 1
			cr = draw_shaded_text(cr, "Inbox", blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
			blx = 2
			bly = bly + 12
			cr = draw_shaded_text(cr, "No New Messages In", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			cairo_text_extents(cr, "No New Messages In ", extents)
			blx = blx + extents.x_advance + 1
			cr = draw_shaded_text(cr, "Inbox2", blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
			blx = 2
			bly = bly + 12
			cr = draw_shaded_text(cr, "(", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			cairo_text_extents(cr, "(", extents)
			blx = blx + extents.x_advance
			cr = draw_shaded_text(cr, "2", blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
			cairo_text_extents(cr, "2", extents)
			blx = blx + extents.x_advance
			cr = draw_shaded_text(cr, ") New Messages In", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			cairo_text_extents(cr, ") New Messages In ", extents)
			blx = blx + extents.x_advance + 1
			cr = draw_shaded_text(cr, "Inbox3", blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
			cairo_text_extents(cr, "Inbox3", extents)
			blx = blx + extents.x_advance + 1
			cr = draw_shaded_text(cr, " From", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			cairo_text_extents(cr, " From ", extents)
			blx = blx + extents.x_advance + 1
			cr = draw_shaded_text(cr, "CrunchBang Linux Forums Mailer <noreply@crunchbanglinux.org>", blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
			blx = 2
			bly = bly + 12
			cr = draw_shaded_text(cr, '"', blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			cairo_text_extents(cr, '"', extents)
			blx = blx + extents.x_advance
			cr = draw_shaded_text(cr, "Reply to topic: 'The New Monster Conky Thread'", blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
			cairo_text_extents(cr, "Reply to topic: 'The New Monster Conky Thread'", extents)
			blx = blx + extents.x_advance
			cr = draw_shaded_text(cr, '"', blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			blx = 2
			bly = bly + 12
			cr = draw_shaded_text(cr, "No New Messages In", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			cairo_text_extents(cr, "No New Messages In ", extents)
			blx = blx + extents.x_advance + 1
			cr = draw_shaded_text(cr, "Inbox4", blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
			blx = 2
			bly = bly + 12
			cr = draw_shaded_text(cr, "No New Messages In", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			cairo_text_extents(cr, "No New Messages In ", extents)
			blx = blx + extents.x_advance + 1
			cr = draw_shaded_text(cr, "Inbox5", blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
		end --# if scrot_mode == false then
		--#### End Evolution Email Data ####
		--#################################################
		cairo_select_font_face (cr, font, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL)
		bly = bly + 4
		cr = draw_horizontal_divider(cr, 1, bly, conky_window.width, bly, 1, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		--#################################################
		if (math.mod((updates - 6), 30) == 0) or (iCalendar_data == nil) then
			local iCalendar_call = io.popen("python /path/to/conky_iCalendar_2_lua.py")
			local iCalendar_output = iCalendar_call:read("*a")
			iCalendar_call:close()
			iCalendar_call = nil
			local iCalendar_function = assert(loadstring(iCalendar_output))
			iCalendar_data = iCalendar_function()
		end --# if math.mod((updates - 6), 30) == 0 then
		if type(iCalendar_data) == "table" then
			if type(iCalendar_data[1][1]) == "table" and #iCalendar_data[1][1] > 1 then
				for i = 1, #iCalendar_data[1] do
					for j = 1, #iCalendar_data[1][i] do
						if j == 1 then
							blx = 2
							bly = bly + 12
							cr = draw_shaded_text(cr, iCalendar_data[1][i][j][1], blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
							cairo_text_extents(cr, iCalendar_data[1][i][j][1].." ", extents)
							blx = blx + extents.x_advance
							cr = draw_shaded_text(cr, '-', blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
							cairo_text_extents(cr, '- ', extents)
							blx = blx + extents.x_advance
							cr = draw_shaded_text(cr, iCalendar_data[1][i][j][2], blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
						else
							blx = 2
							bly = bly + 12
							cr = draw_shaded_text(cr, iCalendar_data[1][i][j][1], blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
							cairo_text_extents(cr, iCalendar_data[1][i][j][1].." ", extents)
							blx = blx + extents.x_advance
							cr = draw_shaded_text(cr, '->', blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
							cairo_text_extents(cr, '-> ', extents)
							blx = blx + extents.x_advance
							if iCalendar_data[1][i][j][3][1]:sub(1,1) == "-" then
								cr = draw_shaded_text(cr, iCalendar_data[1][i][j][2].." |", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
								cairo_text_extents(cr, iCalendar_data[1][i][j][2].." |", extents)
							else
								cr = draw_shaded_text(cr, iCalendar_data[1][i][j][2].." |", blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
								cairo_text_extents(cr, iCalendar_data[1][i][j][2].." | ", extents)
							end --# if iCalendar_data[1][i][j][3][1]:sub(1,1) == "-" then
							blx = blx + extents.x_advance
							cr = draw_shaded_text(cr, iCalendar_data[1][i][j][3][1], blx, bly, iCalendar_data[1][i][j][3][2], iCalendar_data[1][i][j][3][3])
							cairo_text_extents(cr, iCalendar_data[1][i][j][3][1].." ", extents)
							blx = blx + extents.x_advance
							cr = draw_shaded_text(cr, '|', blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
							cairo_text_extents(cr, '| ', extents)
							blx = blx + extents.x_advance
							cr = draw_shaded_text(cr, iCalendar_data[1][i][j][4], blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
						end --# if j == 1 then
					end --# for j = 1, #iCalendar_data[1][i] do
					cairo_select_font_face (cr, font, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL)
					bly = bly + 4
					cr = draw_horizontal_divider(cr, 1, bly, conky_window.width, bly, 1, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
				end --# for i = 1, #iCalendar_data[1] do
			else
				blx = 2
				bly = bly + 12
				cr = draw_shaded_text(cr, iCalendar_data[1][1][1], blx, bly, {color0_R,color0_G,color0_B,color0_A}, {color3_R,color3_G,color3_B,color3_A})
				bly = bly + 4
				cr = draw_horizontal_divider(cr, 1, bly, conky_window.width, bly, 1, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
			end --# if #iCalendar_data[1] > 1 then
			cairo_text_extents(cr, iCalendar_data[2][1][2], extents)
			local calendarWidth = extents.x_advance
			local middleCalStart = (conkyWidth - calendarWidth)/2 --# center line 1 in window
			local leftCalStart = (middleCalStart - calendarWidth)/2
			local rightCalStart = (middleCalStart + calendarWidth + leftCalStart)
			local startArray = {leftCalStart, middleCalStart, rightCalStart}
			bly2 = bly
			for i = 1, 3 do
				bly = bly2
				for j = 1, #iCalendar_data[2][i] do
					blx = 0
					
					if j == 1 then
						bly = bly + 12
						cairo_text_extents(cr, iCalendar_data[2][i][1], extents)
						blx = (startArray[i] + ((calendarWidth - extents.x_advance)/2))
						cr = draw_shaded_text(cr, iCalendar_data[2][i][1], blx, bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
					elseif j == 2 then
						bly = bly + 12
						cr = draw_shaded_text(cr, iCalendar_data[2][i][2], startArray[i], bly, {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
					else
						blx = startArray[i]
						bly = bly + 12
						for k = 1, # iCalendar_data[2][i][j] do
							if iCalendar_data[2][i][j][k][2] == "Bold" then
								cairo_select_font_face (cr, font, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_BOLD)
							end --# if iCalendar_data[2][i][j][2] == "Bold" then
							cr = draw_shaded_text(cr, iCalendar_data[2][i][j][k][1], blx, bly, iCalendar_data[2][i][j][k][3], iCalendar_data[2][i][j][k][4])
							cairo_text_extents(cr, iCalendar_data[2][i][j][k][1]..' ', extents)
							blx = blx + extents.x_advance
							if iCalendar_data[2][i][j][k][2] == "Bold" then
								cairo_select_font_face (cr, font, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL)
							end --# if iCalendar_data[2][i][j][2] == "Bold" then
						end --# for k = 1, # iCalendar_data[2][i][j] do
					end --# if j = 1 then
				end --# for j = 1, #iCalendar_data[2][i] do
			end --# for i = 1, 3 do
		end --# if type(iCalendar_data) == "table" then
		--#################################################################################################
		cairo_select_font_face (cr, font, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL)
		cairo_set_font_size (cr, bfSize)
		--#################################################################################################
		button_name = "resetInbox1"
		--##############################################################
		if (localx >= buttons[button_name]["bl_x"]) and (localx <= (buttons[button_name]["bl_x"] + buttons[button_name]["width"])) and (localy <= buttons[button_name]["bl_y"]) and (localy >= (buttons[button_name]["bl_y"] - buttons[button_name]["height"])) and (buttons[button_name]["state"] == 0) then
			buttons[button_name]["state"] = 1
		else
			buttons[button_name]["state"] = 0
		end --# if (localx >= buttons[button_name]["bl_x"]) and (localx <= (buttons[button_name]["bl_x"] + buttons[button_name]["width"])) and (localy <= buttons[button_name]["bl_y"]) and (localy >= (buttons[button_name]["bl_y"] - buttons[button_name]["height"])) and (buttons[button_name]["state"] == 0) then
		--##############################################################
		if buttons[button_name]["state"] == 1 then
			color = {color2_R,color2_G,color2_B,color2_A} --# if button is clicked on it will be drawn this color
			fill = 0
			os.execute ("dbus-send --dest=org.freedesktop.DBus \"/org/freedesktop/sample/object/name\" \"evolution.conky.reset\" string:\"Inbox\"")
		else
			color = {color0_R,color0_G,color0_B,color0_A}
			fill = 0
		end --# if buttons[button_name]["state"] == 1 then
		if scrot_mode == true then
			cr = draw_shaded_text(cr, "Inbox", (buttons[button_name]["bl_x"] + 1),(buttons[button_name]["bl_y"] - 3), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		else
			cr = draw_shaded_text(cr, "Inbox", (buttons[button_name]["bl_x"] + 1),(buttons[button_name]["bl_y"] - 3), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		end --# if scrot_mode == true
		--##### draw the button ####################
		cr = buttondraw(cr, buttons[button_name]["bl_x"], buttons[button_name]["bl_y"], buttons[button_name]["height"], buttons[button_name]["width"], color, fill)
		--#################################################################################################
		button_name = "resetInbox2"
		--##############################################################
		if (localx >= buttons[button_name]["bl_x"]) and (localx <= (buttons[button_name]["bl_x"] + buttons[button_name]["width"])) and (localy <= buttons[button_name]["bl_y"]) and (localy >= (buttons[button_name]["bl_y"] - buttons[button_name]["height"])) and (buttons[button_name]["state"] == 0) then
			buttons[button_name]["state"] = 1
		else
			buttons[button_name]["state"] = 0
		end --# if (localx >= buttons[button_name]["bl_x"]) and (localx <= (buttons[button_name]["bl_x"] + buttons[button_name]["width"])) and (localy <= buttons[button_name]["bl_y"]) and (localy >= (buttons[button_name]["bl_y"] - buttons[button_name]["height"])) and (buttons[button_name]["state"] == 0) then
		--##############################################################
		if buttons[button_name]["state"] == 1 then
			color = {color2_R,color2_G,color2_B,color2_A} --# if button is clicked on it will be drawn this color
			fill = 0
			os.execute ("dbus-send --dest=org.freedesktop.DBus \"/org/freedesktop/sample/object/name\" \"evolution.conky.reset\" string:\"Inbox2\"")
		else
			color = {color0_R,color0_G,color0_B,color0_A}
			fill = 0
		end --# if buttons[button_name]["state"] == 1 then
		if scrot_mode == true then
			cr = draw_shaded_text(cr, "Inbox2", (buttons[button_name]["bl_x"] + 1),(buttons[button_name]["bl_y"] - 3), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		else
			cr = draw_shaded_text(cr, "Inbox2", (buttons[button_name]["bl_x"] + 1),(buttons[button_name]["bl_y"] - 3), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		end --# if scrot_mode == true
		--##### draw the button ####################
		cr = buttondraw(cr, buttons[button_name]["bl_x"], buttons[button_name]["bl_y"], buttons[button_name]["height"], buttons[button_name]["width"], color, fill)
		--#################################################################################################
		button_name = "resetInbox3"
		--##############################################################
		if (localx >= buttons[button_name]["bl_x"]) and (localx <= (buttons[button_name]["bl_x"] + buttons[button_name]["width"])) and (localy <= buttons[button_name]["bl_y"]) and (localy >= (buttons[button_name]["bl_y"] - buttons[button_name]["height"])) and (buttons[button_name]["state"] == 0) then
			buttons[button_name]["state"] = 1
		else
			buttons[button_name]["state"] = 0
		end --# if (localx >= buttons[button_name]["bl_x"]) and (localx <= (buttons[button_name]["bl_x"] + buttons[button_name]["width"])) and (localy <= buttons[button_name]["bl_y"]) and (localy >= (buttons[button_name]["bl_y"] - buttons[button_name]["height"])) and (buttons[button_name]["state"] == 0) then
		--##############################################################
		if buttons[button_name]["state"] == 1 then
			color = {color2_R,color2_G,color2_B,color2_A} --# if button is clicked on it will be drawn this color
			fill = 0
			os.execute ("dbus-send --dest=org.freedesktop.DBus \"/org/freedesktop/sample/object/name\" \"evolution.conky.reset\" string:\"Inbox3\"")
		else
			color = {color0_R,color0_G,color0_B,color0_A}
			fill = 0
		end --# if buttons[button_name]["state"] == 1 then
		if scrot_mode == true then
			cr = draw_shaded_text(cr, "Inbox3", (buttons[button_name]["bl_x"] + 1),(buttons[button_name]["bl_y"] - 3), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		else
			cr = draw_shaded_text(cr, "Inbox3", (buttons[button_name]["bl_x"] + 1),(buttons[button_name]["bl_y"] - 3), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		end --# if scrot_mode == true
		--##### draw the button ####################
		cr = buttondraw(cr, buttons[button_name]["bl_x"], buttons[button_name]["bl_y"], buttons[button_name]["height"], buttons[button_name]["width"], color, fill)
		--#################################################################################################
		button_name = "resetInbox4"
		--##############################################################
		if (localx >= buttons[button_name]["bl_x"]) and (localx <= (buttons[button_name]["bl_x"] + buttons[button_name]["width"])) and (localy <= buttons[button_name]["bl_y"]) and (localy >= (buttons[button_name]["bl_y"] - buttons[button_name]["height"])) and (buttons[button_name]["state"] == 0) then
			buttons[button_name]["state"] = 1
		else
			buttons[button_name]["state"] = 0
		end --# if (localx >= buttons[button_name]["bl_x"]) and (localx <= (buttons[button_name]["bl_x"] + buttons[button_name]["width"])) and (localy <= buttons[button_name]["bl_y"]) and (localy >= (buttons[button_name]["bl_y"] - buttons[button_name]["height"])) and (buttons[button_name]["state"] == 0) then
		--##############################################################
		if buttons[button_name]["state"] == 1 then
			color = {color2_R,color2_G,color2_B,color2_A} --# if button is clicked on it will be drawn this color
			fill = 0
			os.execute ("dbus-send --dest=org.freedesktop.DBus \"/org/freedesktop/sample/object/name\" \"evolution.conky.reset\" string:\"Inbox4\"")
		else
			color={color0_R,color0_G,color0_B,color0_A}
			fill=0
		end --# if buttons[button_name]["state"] == 1 then
		if scrot_mode == true then
			cr = draw_shaded_text(cr, "Inbox4", (buttons[button_name]["bl_x"] + 1),(buttons[button_name]["bl_y"] - 3), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		else
			cr = draw_shaded_text(cr, "Inbox4", (buttons[button_name]["bl_x"] + 1),(buttons[button_name]["bl_y"] - 3), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		end --# if scrot_mode == true
		--##### draw the button ####################
		cr = buttondraw(cr, buttons[button_name]["bl_x"], buttons[button_name]["bl_y"], buttons[button_name]["height"], buttons[button_name]["width"], color, fill)
		--#################################################################################################
		button_name = "resetInbox5"
		--##############################################################
		if (localx >= buttons[button_name]["bl_x"]) and (localx <= (buttons[button_name]["bl_x"] + buttons[button_name]["width"])) and (localy <= buttons[button_name]["bl_y"]) and (localy >= (buttons[button_name]["bl_y"] - buttons[button_name]["height"])) and (buttons[button_name]["state"] == 0) then
			buttons[button_name]["state"] = 1
		else
			buttons[button_name]["state"] = 0
		end --# if (localx >= buttons[button_name]["bl_x"]) and (localx <= (buttons[button_name]["bl_x"] + buttons[button_name]["width"])) and (localy <= buttons[button_name]["bl_y"]) and (localy >= (buttons[button_name]["bl_y"] - buttons[button_name]["height"])) and (buttons[button_name]["state"] == 0) then
		--##############################################################
		if buttons[button_name]["state"] == 1 then
			color = {color2_R,color2_G,color2_B,color2_A} --# if button is clicked on it will be drawn this color
			fill = 0
			os.execute ("dbus-send --dest=org.freedesktop.DBus \"/org/freedesktop/sample/object/name\" \"evolution.conky.reset\" string:\"Inbox5\"")
		else
			color = {color0_R,color0_G,color0_B,color0_A}
			fill = 0
		end --# if buttons[button_name]["state"] == 1 then
		if scrot_mode == true then
			cr = draw_shaded_text(cr, "Inbox5", (buttons[button_name]["bl_x"] + 1),(buttons[button_name]["bl_y"] - 3), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		else
			cr = draw_shaded_text(cr, "Inbox5", (buttons[button_name]["bl_x"] + 1),(buttons[button_name]["bl_y"] - 3), {color1_R,color1_G,color1_B,color1_A}, {color3_R,color3_G,color3_B,color3_A})
		end --# if scrot_mode == true
		--##### draw the button ####################
		cr = buttondraw(cr, buttons[button_name]["bl_x"], buttons[button_name]["bl_y"], buttons[button_name]["height"], buttons[button_name]["width"], color, fill)
		--#################################################################################################
	end --# if updates > 5
	if previousEpoch < newEpoch then
		previousEpoch = newEpoch
	else
		print("error: epoch check warning old:"..previousEpoch.." new:"..newEpoch)
	end --# if previousEpoch < newEpoch then
	cairo_destroy(cr)
	cairo_surface_destroy(cs)
	cr = nil
	cs = nil
	local gcollect = collectgarbage ("collect")
end-- end main function

function drawBar(CR, xPos,yPos,barHeight,barLength,barAlpha,valueType,inputVariable,inputParameter)
	--cairo_set_line_join (cr, CAIRO_LINE_JOIN_MITER) --# in case of wacky defaults
	--cairo_set_antialias (cr, CAIRO_ANTIALIAS_NONE) --# don't antialias outline it is too thin and gets blurred
	---#####
	local barAlpha = (barAlpha)/255
	---#####
	local inputValue = nil
	if valueType == 0 then -- get value from conky
		--print("test")
		inputValue = tonumber(conky_parse("${"..inputVariable.." "..inputParameter.."}"))
		--print(inputValue)
	else
		inputValue = tonumber(inputVariable)
	end --# if valueType == 0
	--SETTINGS FOR CPU INDICATOR BAR
	local barLineWidth = 1
	local bar_length = barLength - (barLineWidth)
	local bar_HEIGHT = barHeight  - (barLineWidth)
	--# draw shade
	cairo_set_line_width (CR, barLineWidth)
	cairo_set_source_rgba (CR,color3_R,color3_G,color3_B,color3_A)
	cairo_rectangle (CR,(xPos + 1),(yPos + 1),barLength,-barHeight)
	cairo_stroke (CR)
	--# draw outline
	cairo_set_source_rgba (CR,color1_R,color1_G,color1_B,color1_A)
	cairo_rectangle (CR,(xPos),(yPos),barLength,-barHeight)
	cairo_stroke (CR)
	--# draw indicator
	cairo_set_source_rgba (CR,color1_R,color1_G,color1_B,barAlpha)--set indicator color
	local max_Value=100
	local scale = bar_length/max_Value
	local indicator_length = scale*inputValue
	cairo_rectangle (CR,(xPos + (barLineWidth/2)),(yPos - (barLineWidth/2)),indicator_length,-bar_HEIGHT)
	cairo_fill (CR)
	return CR
	--cairo_set_antialias (cr, CAIRO_ANTIALIAS_DEFAULT)
	--####
end

function draw_graph(CR, data,max_Value,table_Length,BLX,BLY,ALPHA,WIDTH,cHeight) --# only works with integer BLX and BLY
	--cairo_set_line_join (cr, CAIRO_LINE_JOIN_MITER) --# in case of wacky defaults
	--cairo_set_antialias (cr, CAIRO_ANTIALIAS_NONE) --# don't antialias outline it is too thin and gets blurred
	--# Draw Outline
	local outline_lineWidth = 1
	cairo_set_line_width (CR,outline_lineWidth)
	local graphWidth = (table_Length*WIDTH) + 1
	local outlineWidth = graphWidth
	local outlineHeight = cHeight + 1
	--# draw shade
	cairo_set_source_rgba (CR,color3_R,color3_G,color3_B,color3_A)
	cairo_rectangle (CR,(BLX + 1),(BLY + 1),outlineWidth,-outlineHeight)
	cairo_stroke (CR)
	--# draw outline
	cairo_set_source_rgba (CR,color1_R,color1_G,color1_B,color1_A)
	cairo_rectangle (CR,(BLX),(BLY),outlineWidth,-outlineHeight)
	cairo_stroke (CR)
	--# Draw Graph
	cairo_set_line_width (CR,WIDTH)
    for i=1,table_Length do
		--# Get gradient color
		local Red,Green,Blue = linear_gradient({color4_R,color4_G,color4_B},{color5_R,color5_G,color5_B}, graphWidth, (graphWidth - i))
		cairo_set_source_rgba (CR,Red,Green,Blue,ALPHA)
        --# calculate bar height
        local bar_height = (data[i]/max_Value)*cHeight
        --# set start position for each bar, and modify with the value of i
		cairo_move_to (CR,((BLX) + (WIDTH/2) + ((i - 1)*WIDTH)), (BLY - 1))
        --# draw relative line, y becomes equal to bar height and must be negative to draw up
        cairo_rel_line_to (CR, 0, (bar_height*-1))
        --# draw the line
        cairo_stroke (CR)
    end --# i=1,table_length do
    return CR
	--cairo_set_antialias (cr, CAIRO_ANTIALIAS_DEFAULT)
end--of function draw_graph

function draw_line_graph(CR, data,max_Value,table_Length,BLX,BLY,ALPHA,WIDTH,cHeight) --# only works with integer BLX and BLY
	--cairo_set_line_join (cr, CAIRO_LINE_JOIN_MITER) --# in case of wacky defaults
	--cairo_set_antialias (cr, CAIRO_ANTIALIAS_NONE) --# don't antialias outline it is too thin and gets blurred
	--# Draw Outline
	local outline_lineWidth = 1
	cairo_set_line_width (CR,outline_lineWidth)
	local graphWidth = (table_Length*WIDTH) + 1
	local outlineWidth = graphWidth
	local outlineHeight = cHeight + 1
	--# draw shade
	cairo_set_source_rgba (CR,color3_R,color3_G,color3_B,color3_A)
	cairo_rectangle (CR,(BLX + 1),(BLY + 1),outlineWidth,-outlineHeight)
	cairo_stroke (CR)
	--# draw outline
	cairo_set_source_rgba (CR,color1_R,color1_G,color1_B,color1_A)
	cairo_rectangle (CR,(BLX),(BLY),outlineWidth,-outlineHeight)
	cairo_stroke (CR)
	--# Draw Graph
	cairo_set_line_width (CR,1)
	--# move to start position
	local bar_height = ((data[1]/max_Value)*cHeight) + 1
	cairo_move_to (CR,((BLX)+(WIDTH/2)+((1-1)*WIDTH)),(BLY - bar_height))
    for i=1,table_Length do
		--# Get gradient color
		local Red,Green,Blue = linear_gradient({color4_R,color4_G,color4_B},{color5_R,color5_G,color5_B}, graphWidth, (graphWidth - i))
		cairo_set_source_rgba (CR,Red,Green,Blue,ALPHA)
        --# calculate data y positon
        local bar_height = ((data[i]/max_Value)*cHeight) + 1
        --# draw line from last point to current point
        cairo_line_to (CR, ((BLX)+(WIDTH/2)+((i-1)*WIDTH)),(BLY - bar_height))
        cairo_stroke (CR)
        --# move to current point
        cairo_move_to (CR,((BLX)+(WIDTH/2)+((i-1)*WIDTH)),(BLY - bar_height))
    end --# i=1,table_length do
    return CR
	--cairo_set_antialias (cr, CAIRO_ANTIALIAS_DEFAULT)
end--of function draw_graph

function linear_gradient(Left_Color, Right_Color, Length, Position)
	local positionPercent = Position/Length
	local red = Right_Color[1] + (math.abs(Right_Color[1] - Left_Color[1]) * positionPercent)
	local green = Right_Color[2] + (math.abs(Right_Color[2] - Left_Color[2]) * positionPercent)
	local blue = Right_Color[3] + (math.abs(Right_Color[3] - Left_Color[3]) * positionPercent)
	return red,green,blue
end --# function linear_gradient()

function draw_shaded_text(CR, text, xpos, ypos, COLOR, shade_color)
	cairo_set_source_rgba (CR,shade_color[1],shade_color[2],shade_color[3],shade_color[4]) --# set shade color
	cairo_move_to (CR,(xpos + 1),(ypos + 1))
	cairo_show_text (CR,text)
	cairo_stroke (CR)
	cairo_set_source_rgba (CR,COLOR[1],COLOR[2],COLOR[3],COLOR[4]) --# set font color
	cairo_move_to (CR,xpos,ypos)
	cairo_show_text (CR,text)
	cairo_stroke (CR)
	return CR
end --# function cr = cr = draw_shaded_text(cr, cr, )

function draw_horizontal_divider(CR, xpos, ypos, xend, yend, lineWidth, COLOR, shade_color)
	--cairo_set_antialias (cr, CAIRO_ANTIALIAS_NONE) --# don't antialias outline it is too thin and gets blurred
	cairo_set_line_width (CR,lineWidth)
	--# draw shade
	cairo_set_source_rgba (CR,shade_color[1],shade_color[2],shade_color[3],shade_color[4]) --# set shade color
	cairo_move_to (CR, (xpos + 1), (ypos + 1))
	cairo_line_to (CR,(xend + 1), (yend + 1))
	cairo_stroke (CR)
	--# draw line
	cairo_set_source_rgba (CR,COLOR[1],COLOR[2],COLOR[3],COLOR[4]) --# set color
	cairo_move_to (CR, xpos, ypos)
	cairo_line_to (CR, xend, yend)
	cairo_stroke (CR)
	--cairo_set_antialias (cr, CAIRO_ANTIALIAS_DEFAULT)
	return CR
end --# function cr = cr = draw_horizontal_divider(cr,cr, )

function image(im)--#################################################################
	local x = nil
	x = (im.x or 0)
	local y = nil
	y = (im.y or 0)
	local w = nil
	w = (im.w or default_image_width)
	local h = nil
	h = (im.h or default_image_height)
	local file = nil
	file = tostring(im.file)
	if file == nil then
		print("set image file") 
	end --# if file == nil then
	--############################################
	local show = imlib_load_image(file)
	if show == nil then
		return 
	end --# if show == nil then
	imlib_context_set_image(show)
	if w == 0 then 
		WIDTH = imlib_image_get_width() 
	else
		WIDTH = w
	end --# if w == 0 then 
	if h == 0 then
		HEIGHT = imlib_image_get_height() 
	else
		HEIGHT = h
	end --# if h == 0 then
	local scaled = imlib_create_cropped_scaled_image(0, 0, imlib_image_get_width(), imlib_image_get_height(), WIDTH, HEIGHT)
	imlib_free_image_and_decache()
	imlib_context_set_image(scaled)
	imlib_render_image_on_drawable(x, y)
	imlib_free_image_and_decache()
	show = nil
	WIDTH = nil
	HEIGHT = nil
	x = nil
	y = nil
	w = nil
	h = nil
	file = nil
	scaled = nil
end --# function image(im) #################################################################

--[[
function rgb_to_r_g_b(COLOR,ALPHA)
	return ((COLOR / 0x10000) % 0x100) / 255., ((COLOR / 0x100) % 0x100) / 255., (COLOR % 0x100) / 255., ALPHA
end
]]

function draw_bg(CR)
	--# "corner_r" is the radius, in pixels, of the rounded corners. If you don't want rounded corners, use 0.
	local corner_r = 0
	local v = 0
	--######
	local w = conky_window.width
	local h = conky_window.height
	cairo_move_to(CR,corner_r,0)
	cairo_line_to(CR,w-corner_r,0)
	cairo_curve_to(CR,w,0,w,0,w,corner_r)
	cairo_line_to(CR,w,h+v-corner_r)
	cairo_curve_to(CR,w,h+v,w,h+v,w-corner_r,h+v)
	cairo_line_to(CR,corner_r,h+v)
	cairo_curve_to(CR,0,h+v,0,h+v,0,h+v-corner_r)
	cairo_line_to(CR,0,corner_r)
	cairo_curve_to(CR,0,0,0,0,corner_r,0)
	cairo_close_path(CR)
	local bgColor = assert(loadfile("/path/to/currentBGLua_log.txt"))()
	cairo_set_source_rgba(CR,bgColor[1],bgColor[2],bgColor[3],bgColor[4])
	cairo_fill(CR)
	return CR
end --# function draw_bg()

--#### Button code based on interactive conky by mrpeachy ####
--#### http://crunchbanglinux.org/forums/topic/18419/interactive-conky/ ####
--# button drawing function
function buttondraw(CR, BLX,BLY,HEIGHT,WIDTH,COLOR,FILL)
	--cairo_set_line_join (cr, CAIRO_LINE_JOIN_MITER) --# in case of wacky defaults
	--cairo_set_antialias (cr, CAIRO_ANTIALIAS_NONE) --# don't antialias outline it is too thin and gets blurred
	cairo_set_line_width (CR,2)
	cairo_set_source_rgba (CR,color3_R,color3_G,color3_B,color3_A)
	cairo_rectangle (CR,(BLX + 1),(BLY + 1),WIDTH,-HEIGHT)
	if FILL == 0 then
		cairo_stroke (CR)
	elseif FILL == 1 then
		cairo_fill (CR)
	end --# if FILL == 0
	cairo_set_source_rgba (CR,COLOR[1],COLOR[2],COLOR[3],COLOR[4])
	cairo_rectangle (CR,BLX,BLY,WIDTH,-HEIGHT)
	if FILL == 0 then
		cairo_stroke (CR)
	elseif FILL == 1 then
		cairo_fill (CR)
	end --# if FILL == 0
	return CR
	--cairo_set_antialias (cr, CAIRO_ANTIALIAS_DEFAULT) --# reset antialias method to default
end--of buttondraw function

function conky_start_xdotool()
    os.execute("pkill -f \"xdotool search --sync --classname conky behave %@ mouse-click getmouselocation\"")
    os.execute("xdotool search --sync --classname 'conky' behave %@ mouse-click getmouselocation 2> /dev/null >> /tmp/"..tempFileName.." &")
end --# function conky_start_xdotool()

function conky_stop_xdotool()
	local stdout = io.popen("ps --no-headers -C conky -O cmd")
	local conkyCount = 0
	local rconkyData = stdout:read("*l")
	while rconkyData ~= nil do
		if ((string.find(rconkyData,"conkyrc_systemConky_2core") ~= nil) or (string.find(rconkyData,"conkyrc_conkyWeather_arclance") ~= nil)) then --# is one of the conkys to check for (could use something generic like "button" here)
			print("found")
			conkyCount = conkyCount + 1
		end --# if string.find("conkyrc_systemConky") ~= nil then
		rconkyData = stdout:read("*l")
	end --# while rconkyData ~= nil do
	stdout:close()
    if conkyCount == 1 then
		print("killing xdotool")
		os.execute("pkill -f \"xdotool search --sync --classname conky behave %@ mouse-click getmouselocation\"")
	end --# if conkyCount == 1 then
end --# function conky_stop_xdotool()

function clickfunction()
	--################################################################################################
	if tonumber(conky_parse('${updates}')) < 7 then
		local s = nil
		local f = io.popen("xwininfo -all -int -tree -name '"..conkyWindowTitle.."'")
		local winData = f:read("*a")
		f:close()
		f = nil
		s,f,xdotool_window_number = string.find(winData,"Parent%swindow%sid.%s(%d+)%s")
		s,f,abstlx = string.find(winData,"Absolute%supper.left%sX%p%s*(%p*%d*)")
		s,f,abstly = string.find(winData,"Absolute%supper.left%sY%p%s*(%p*%d*)")
		print("abstlx: "..tostring(abstlx).." abstly: "..tostring(abstly).." winNum: "..tostring(xdotool_window_number))
	end --# if tonumber(conky_parse('${updates}')) == 1 then
	--#################################################################################################
    --# start click logging and calculations ##########################################
    local s = nil
    --## Get Current Mouse Location
	local f = io.popen("xdotool getmouselocation 2> /dev/null")
	local mousenow = f:read("*a")
	f:close()
	f = nil
	local mousenowx = nil
	local mousenowy = nil
	local mousex = nil
	local mousey = nil
	if ((mousenow == nil) or (mousenow == "")) then --# check for failure of xdotool getmouselocation command (crashes script but usually only lasts one or two updates)
		print("error: could not get mouse location")
		mousenowx = "0"
		mousenowy = "0"
		mousex = "0"
		mousey = "0"
	elseif string.find(mousenow, xdotool_window_number) ~= nil then --# check if mouse is in this conky window
		--print("debug: mouse is in this window")
		s,f,mousenowx = string.find(mousenow,"x%p(%d*)%s")
		s,f,mousenowy = string.find(mousenow,"y%p(%d*)%s")
		--#### Get Click Location
		f = io.popen("tail --lines=1 /tmp/"..tempFileName)
		local click = f:read("*a")
		f:close()
		f = nil
		if ((click == nil) or (click == "")) then --# xdotool log file does not exist or is empty (prevents crash before first click or if you specify the wrong file)
			print("error: could not read xdotool logfile")
			mousex = "0"
			mousey = "0"
		elseif (click:gsub("\n","") == "Command failed.") then --# this is does not crash xdotool (probably a communication error between xdotool and some part of the system)
			print("error: xdotool error")
			mousex = "0"
			mousey = "0"
		else --# no problems reading the xdotool log file
			if string.find(click, xdotool_window_number) ~= nil then --# last click was in this window
				print("debug: click was in this window")
				s,f,mousex = string.find(click,"x%p(%d*)")
				s,f,mousey = string.find(click,"y%p(%d*)")
				--### Reset xdotool log file
				f = io.open("/tmp/"..tempFileName,"w")
				f:write("x:0 y:0 screen:0 window:0\n")
				f:close()
				f = nil
			else --# last click was not in this window
				--print("debug: click not in this window")
				mousex = "0"
				mousey = "0"
			end --# if string.find(click, xdotool_window_number) ~= nil then
		end --# if ((click == nil) or (click == "")) then
	else --# mouse not in this window
		mousenowx = "0"
		mousenowy = "0"
		mousex = "0"
		mousey = "0"
	end --# if string.find(mousenow, xdotool_window_number) then
	local localx = tonumber(mousex) - abstlx
	local localy = tonumber(mousey) - abstly
	local localnowx = tonumber(mousenowx) - abstlx
	local localnowy = tonumber(mousenowy) - abstly
    --# END CLICK CALCULATIONS #################################
    return localx,localy,localnowx,localnowy
end --# function clickfunction()
