#!/usr/bin/env python2.7
# -*- coding: utf-8 -*-
"""
Name: conky_icalendar_2.py
Author: arclance
Date_Created:2012-01-10
Date_Updated:2012-07-13
	
Reads the evolution calendar(or another ical calendar) using calcurse and format the output for dispaly in conky.
Does not work with recurring appointments yet (need to write appointment sorting function first).

Requires
	icalendar python package v3.0 or higher | http://pypi.python.org/pypi/icalendar/3.0.1b2#downloads
	time
	calendar
	####subprocess

Changelog
v3.1 2012-07-13
	Remove old non lua code, the old three calendar code was not portable to other systems.

v3.0 2012-07-12
	begin adding output for dispaly with lua script
	update time_until to use epoch time it is more accurate and works with appointments that start and end on different days

v2.X.0 2012-05-11 to 2012-05-22
	allow for calendar to use font other than default font
	add countdown color change for appointments that have not ended yet
	made countdown colors easier to change
	2012-05-14
	added support for exceptions for WEEKLY recurring events (days in the reccurance pattern when an event does not occur)
	2012-05-22
	use calendar module instead of cal to get raw calendars
	added support for exceptions for DAILY recurring events (days in the reccurance pattern when an event does not occur)
	fix exception handling for events with only one exception
	added support for exceptions for YEARLY recurring events (days in the reccurance pattern when an event does not occur)
	2012-06-01
	fix highlighting of single digit days in new script
	
	Beta of three calendar output
		fix month offset adjustment (now works across multiple years)
		finish Month-Year header alignment for each month (will only work with a font with the same character width as DejaVu Sans Mono:size=10)
		2012-05-14
		fix alignment for multiple calendars

v2.6.0 2012-05-11 to 2012-05-12
	fix highlighting of current day if there is no appointment
	improve alignment of calendar columns
	remove extra color changes from calendar (small cpu use decrease for conky)
	
v2.5.2 2012-05-11
	improve handling of YEARLY recurring events (allow for leap years)

v2.5.1 2012-05-11
	fix for displaying non-ascii characters in conky (worked in terminal before this makes it work in conky)

v2.5 2012-05-10
	Add support for DAILY, WEEKLY, and YEARLY recurring events

v2.0 2012-05-09
	replace calcurse with icalendar python module
	rewrite appointment display
	fixed to12Hour (12 PM was showed as 12 AM | 12 AM would have been displayed as 00 AM)
	- 2012-05-10 -
	rewrite calendar display
		fixed entry duplication bug in appointmentDates
"""

import time
from icalendar.cal import Calendar, Event
from icalendar.prop import vDatetime, vDDDTypes, vDDDLists
import calendar

#### user defined variables ####
daysToDisplay = 3 # number of days to display appointments for
icalendarPath = '~/.local/share/evolution/calendar/system/calendar.ics' # location of ics format calendar file, default is the loctaion for evolution but files from other programs should work but are untested
usingDefaultFont = True # if the calendar font is the same as the default font in your .conkyrc set this to True for cpu power savings
defaultColor_Lua = "{color0_R,color0_G,color0_B,color0_A}"
color0_Lua = "{color1_R,color1_G,color1_B,color1_A}"
usingShading = True # Set to True if using draw_shades yes
defaultShadeColor_Lua = "{color3_R,color3_G,color3_B,color3_A}"
secondaryShadeColor_Lua = "{0,0,0,1}"
conkyWidth = 619 # width of conky in pixels | use with a fixed width conky
##### Countdown Colors
currentApptColor_Lua = "{" + str(float(165)/float(255)) + ",0,1,1}"
_30minToApptColor_Lua = "{color6_R,color6_G,color6_B,color6_A}"
_90minToApptColor_Lua = "{color7_R,color7_G,color7_B,color7_A}"
_150minToApptColor_Lua = "{color8_R,color8_G,color8_B,color8_A}"
_240minToApptColor_Lua = "{0," + str(float(100)/float(255)) + ",0,1}"
#####
#### end user defined variables ####

def to12Hour(inputTime):
	Hour = int(inputTime.split(":")[0])
	Minute = int(inputTime.split(":")[1])
	if Hour >= 12: # check for 24 hour format
		if Hour > 12:
			Hour = (Hour - 12) # make 12 hour format
		#endif
		_12HourTime = str(Hour).rjust(2, "0") + ':' + str(Minute).rjust(2, "0") + ' PM' # make sunset time string
	else:
		if Hour == 0:
			Hour = 12
		#endif
		_12HourTime = str(Hour).rjust(2, "0") + ':' + str(Minute).rjust(2, "0") + ' AM' # make sunset time string
	#endif
	return _12HourTime
#enddef	

def timeUntil_Epoch_Lua(secondsFromTimeZero, secondsFromTimeZero_End, dayHappens):
	secondsSinceTimeZero = time.time()	# calculate seconds since the start of today
	if secondsSinceTimeZero > secondsFromTimeZero: # event has already ocurred
		secondsLeft = - secondsFromTimeZero + secondsSinceTimeZero # calculate seconds until the event
		hoursLeft = secondsLeft / 3600 # get hours left
		minutesLeft = (secondsLeft % 3600) / 60 # get minutes left
		hoursLeft = int(hoursLeft)
		minutesLeft = int(minutesLeft)
		if secondsSinceTimeZero < secondsFromTimeZero_End: # Change Color Of Countdown if event has not ended yet
			if usingShading == True:
				timeLeft = '{"-' + str(hoursLeft).rjust(2, "0") + ":" + str(minutesLeft).rjust(2, "0") + '",' + currentApptColor_Lua + ',' + secondaryShadeColor_Lua + '}'
			else:
				timeLeft = "${color " + currentApptColor + "}-" + str(hoursLeft).rjust(2, "0") + ":" + str(minutesLeft).rjust(2, "0") # make timeLeft string
				timeLeft = '{"-' + str(hoursLeft).rjust(2, "0") + ":" + str(minutesLeft).rjust(2, "0") + '",' + currentApptColor_Lua + ',""}'
			#endif
		else:
			timeLeft = '{"-' + str(hoursLeft).rjust(2, "0") + ":" + str(minutesLeft).rjust(2, "0") + '",' + defaultColor_Lua + ',' + defaultShadeColor_Lua + '}'
		#endif
	else:
		secondsLeft = secondsFromTimeZero - secondsSinceTimeZero # calculate seconds until the event
		hoursLeft = secondsLeft / 3600 # get hours left
		minutesLeft = (secondsLeft % 3600) / 60 # get minutes left
		hoursLeft = int(hoursLeft)
		minutesLeft = int(minutesLeft)
		if secondsLeft <= (3600*0.5):
			if usingShading == True:
				timeLeft = '{"' + str(hoursLeft).rjust(2, "0") + ":" + str(minutesLeft).rjust(2, "0") + '",' + _30minToApptColor_Lua + ',' + secondaryShadeColor_Lua + '}'
			else:
				timeLeft = '{"' + str(hoursLeft).rjust(2, "0") + ":" + str(minutesLeft).rjust(2, "0") + '",' + _30minToApptColor_Lua + ',""}'
			#endif
		elif secondsLeft <= (3600*1.5):
			timeLeft = '{"' + str(hoursLeft).rjust(2, "0") + ":" + str(minutesLeft).rjust(2, "0") + '",' + _90minToApptColor_Lua + ',' + defaultShadeColor_Lua + '}'
		elif secondsLeft <= (3600*2.5):
			timeLeft = '{"' + str(hoursLeft).rjust(2, "0") + ":" + str(minutesLeft).rjust(2, "0") + '",' + _150minToApptColor_Lua + ',' + defaultShadeColor_Lua + '}'
		elif secondsLeft <= (3600*4):
			if usingShading == True:
				timeLeft = '{"' + str(hoursLeft).rjust(2, "0") + ":" + str(minutesLeft).rjust(2, "0") + '",' + _240minToApptColor_Lua + ',' + secondaryShadeColor_Lua + '}'
			else:
				timeLeft = '{"' + str(hoursLeft).rjust(2, "0") + ":" + str(minutesLeft).rjust(2, "0") + '",' + _240minToApptColor_Lua + ',""}'		
			#endif
		else:	
			timeLeft = '{"' + str(hoursLeft).rjust(2, "0") + ":" + str(minutesLeft).rjust(2, "0") + '",' + defaultColor_Lua + ',' + defaultShadeColor_Lua + '}'
		#endif
	#endif
	return timeLeft
#enddef

##############################

class eventList:
	
	def __init__(self):
		self._listOfEvents = []
		self._daysToDisplay = daysToDisplay
		self._currentTime = time.time()
		self._currentDate = time.strftime('%Y-%m-%d' ,time.localtime(float(self._currentTime)))
		self._currentDateSplit = self._currentDate.split("-")
		self._currentDateSplit[2] = self._currentDateSplit[2].lstrip("0")
		self._currentWeekday = time.strftime('%a' ,time.localtime(float(self._currentTime)))
		self._checkDates = [self._currentDate]
		for i in xrange(self._daysToDisplay - 1):
			self._checkDates.append(time.strftime('%Y-%m-%d' ,time.localtime(float(self._currentTime + float((i + 1)*3600*24)))))
		#endfor
	#enddef
	
	def appendSingleEvent(self, newEvent):
		self._listOfEvents.append(newEvent)
	#enddef
	
	def appendReccuringEvent_Weekly(self, startDate, startTime, endDate, endTime, recurDays, recurEndDate, recurCount, typeWeekly, listExceptions, summary): # eventList.appendReccuringEvent_Weekly(eventStartDate, eventStartTime, eventEndDate, eventEndTime, eventRecurDays, eventRecurEndDate, eventSummary)
		for i in xrange(len(recurDays)):
			recurDays[i] = str(recurDays[i]).strip().replace("SA","Sat").replace("SU","Sun").replace("MO","Mon").replace("TU","Tue").replace("WE","Wed").replace("TH","Thu").replace("FR","Fri")
		#endfor
		if typeWeekly == 1:
			if len(str(recurEndDate)) < 19:
				recurEndDate_Seconds = time.mktime(time.strptime(str(recurEndDate), "%Y-%m-%d"))
			else:
				recurEndDate_Seconds = time.mktime(time.strptime(str(recurEndDate), "%Y-%m-%d %H:%M:%S"))
			#endif
			if recurEndDate_Seconds > self._currentTime: # only calculate recurring events if the reccurance has not ended yet
				numDays = 0
				timeStartSeconds = dateInSeconds(startDate, startTime)
				timeEndSeconds = dateInSeconds(endDate, endTime)
				checkDateSeconds = float(timeStartSeconds + float(numDays*3600*24))
				dayOfWeek = time.strftime('%a' ,time.localtime(checkDateSeconds))
				while checkDateSeconds < recurEndDate_Seconds:
					if (dayOfWeek in recurDays) and (checkDateSeconds not in listExceptions):
						checkStartDate = time.strftime('%Y-%m-%d',time.localtime(checkDateSeconds))
						checkEndDate = time.strftime('%Y-%m-%d',time.localtime(float(timeEndSeconds + float(numDays*3600*24))))
						self._listOfEvents.append([checkDateSeconds, checkStartDate, startTime, checkEndDate, endTime, summary, "WEEKLY"])
					#endif
					numDays = numDays + 1
					checkDateSeconds = float(timeStartSeconds + float(numDays*3600*24))
					dayOfWeek = time.strftime('%a' ,time.localtime(checkDateSeconds))
				#endwhile
			#endif
		elif typeWeekly == 2:
			numOccurances = 0
			numDays = 0
			timeStartSeconds = dateInSeconds(startDate, startTime)
			timeEndSeconds = dateInSeconds(endDate, endTime)
			while numOccurances < recurCount[0]:
				checkDateSeconds = float(timeStartSeconds + float(numDays*3600*24))
				dayOfWeek = time.strftime('%a' ,time.localtime(checkDateSeconds))
				if (dayOfWeek in recurDays) and (checkDateSeconds not in listExceptions):
					checkStartDate = time.strftime('%Y-%m-%d',time.localtime(checkDateSeconds))
					checkEndDate = time.strftime('%Y-%m-%d',time.localtime(float(timeEndSeconds + float(numDays*3600*24))))
					self._listOfEvents.append([checkDateSeconds, checkStartDate, startTime, checkEndDate, endTime, summary, "WEEKLY"])
					numOccurances = numOccurances + 1
				#endif
				numDays = numDays + 1
			#endwhile
		#endif
	#enddef
	
	def appendReccuringEvent_Daily(self, startDate, startTime, endDate, endTime, recurEndDate, recurCount, recurInterval, typeDaily, listExceptions, summary):
		if typeDaily == 1:
			timeStartSeconds = dateInSeconds(startDate, startTime)
			timeEndSeconds = dateInSeconds(endDate, endTime)
			for i in xrange(0, int(recurCount[0]*recurInterval[0]), recurInterval[0]):
				checkDateSeconds = float(timeStartSeconds + float(i*3600*24))
				checkStartDate = time.strftime('%Y-%m-%d',time.localtime(checkDateSeconds))
				checkEndDate = time.strftime('%Y-%m-%d',time.localtime(float(timeEndSeconds + float(i*3600*24))))
				if checkDateSeconds not in listExceptions:
					self._listOfEvents.append([checkDateSeconds, checkStartDate, startTime, checkEndDate, endTime, summary, "DAILY"])
				#endif
			#endfor
		elif typeDaily == 2:
			timeStartSeconds = dateInSeconds(startDate, startTime)
			timeEndSeconds = dateInSeconds(endDate, endTime)
			for i in xrange(recurCount[0]):
				checkDateSeconds = float(timeStartSeconds + float(i*3600*24))
				checkStartDate = time.strftime('%Y-%m-%d',time.localtime(checkDateSeconds))
				checkEndDate = time.strftime('%Y-%m-%d',time.localtime(float(timeEndSeconds + float(i*3600*24))))
				if checkDateSeconds not in listExceptions:
					self._listOfEvents.append([checkDateSeconds, checkStartDate, startTime, checkEndDate, endTime, summary, "DAILY"])
				#endif
			#endfor
		#endif
	#enddef
	
	def appendReccuringEvent_Monthly(self):
		pass
	#enddef
	
	def appendReccuringEvent_Yearly(self, startDate, startTime, endDate, endTime, recurCount, typeYearly, listExceptions, summary):
		if typeYearly == 1:
			startDateSplit = startDate.split("-")
			endDateSplit = endDate.split("-")
			for i in xrange(recurCount[0]):
				checkStartDate = str(int(startDateSplit[0]) + i) + "-" + startDateSplit[1] + "-" + startDateSplit[2]
				checkEndDate = str(int(endDateSplit[0]) + i) + "-" + endDateSplit[1] + "-" + endDateSplit[2]
				timeStartSeconds = dateInSeconds(checkStartDate, startTime)
				if timeStartSeconds not in listExceptions:
					self._listOfEvents.append([timeStartSeconds, checkStartDate, startTime, checkEndDate, endTime, summary, "YEARLY"])
				#endif
			#endfor
		#endif
	#enddef
	
	def sortList(self):
		self._sortedList = sorted(self._listOfEvents, key=lambda epoch: int(epoch[0]))
	#enddef
	
	def returnList(self):
		return self._listOfEvents
	#enddef
	
	def returnSortedList(self):
		return self._sortedList
	#enddef
	
	def conkyApts_Lua(self):
		previousDate = ""
		outputString = u"{ " # set outputString to empty String
		firstAppointment = True
		##print "conkyApts Test"
		for l in xrange(len(self._sortedList)):
			for i in xrange(len(self._checkDates)):
				checkDateString = self._checkDates[i]
				if (str(self._sortedList[l][1]).strip() == checkDateString):
					if i == 0: # today
						if previousDate != self._sortedList[l][1]:
							dayName = time.strftime('%A' ,time.localtime(float(self._currentTime + float(3600*24*i)))) # get the weekday name
							outputString = outputString + '{ {"Today (' + dayName + ')","' + checkDateString + '"}'
							previousDate = checkDateString
							firstAppointment = False
						#endif
						startTime = self._sortedList[l][2]
						endTime = self._sortedList[l][4]
						eventName = self._sortedList[l][5]
						outputString = outputString + ' , {"' + to12Hour(startTime) + '","' + to12Hour(endTime) + '",' + timeUntil_Epoch_Lua(self._sortedList[l][0], dateInSeconds(self._sortedList[l][3], endTime), i) + ',"' + eventName.replace("$","$$") + '"}'
					elif i == 1: # tommorow
						if previousDate != self._sortedList[l][1]:
							dayName = time.strftime('%A' ,time.localtime(float(self._currentTime + float(3600*24*i)))) # get the weekday name
							if firstAppointment == True:
								outputString = outputString + '{ {"Tommorow (' + dayName + ')","' + checkDateString + '"}'
								firstAppointment = False
							else:
								outputString = outputString + ' } , { {"Tommorow (' + dayName + ')","' + checkDateString + '"}'
							#endif
							previousDate = checkDateString
						#endif
						startTime = self._sortedList[l][2]
						endTime = self._sortedList[l][4]
						eventName = self._sortedList[l][5]
						outputString = outputString + ' , {"' + to12Hour(startTime) + '","' + to12Hour(endTime) + '",' + timeUntil_Epoch_Lua(self._sortedList[l][0], dateInSeconds(self._sortedList[l][3], endTime), i) + ',"' + eventName.replace("$","$$") + '"}'
					else:
						if previousDate != self._sortedList[l][1]:
							dayName = time.strftime('%A' ,time.localtime(float(self._currentTime + float(3600*24*i)))) # get the weekday name
							if firstAppointment == True:
								outputString = outputString + '{ {"' + dayName + '","' + checkDateString + '"}'
								firstAppointment = False
							else:
								outputString = outputString + ' } , { {"' + dayName + '","' + checkDateString + '"}'
							#endif
							previousDate = checkDateString
						#endif
						startTime = self._sortedList[l][2]
						endTime = self._sortedList[l][4]
						eventName = self._sortedList[l][5]
						outputString = outputString + ' , {"' + to12Hour(startTime) + '","' + to12Hour(endTime) + '",' + timeUntil_Epoch_Lua(self._sortedList[l][0], dateInSeconds(self._sortedList[l][3], endTime), i) + ',"' + eventName.replace("$","$$") + '"}'
					#endif	
				#endif	
			#endfor
		#endfor
		if outputString != "{ ":
			return outputString + ' } }'
		else:
			return outputString + '{"No Events"} }'
		#endif
	#enddef
	
	def conkyCalendar_threeMonth_Lua(self, monthOffset):
		calendarList = []
		appointmentDateList = []
		rawLengthList = []
		isCurrentMonth = []
		numYear = int(self._currentDateSplit[0])
		numMonth = (int(self._currentDateSplit[1]) + monthOffset)
		if numMonth < 1:
			numMonth = 12 - (numMonth + 1)
			numYear = numYear - 1
		elif numMonth > 12:
			numMonth = numMonth - 12
			numYear = numYear + 1
		#endif
		calendar.setfirstweekday(6)
		for i in xrange(3):
			if i != 0 and numMonth == 12:
				numYear = numYear + 1
				numMonth = 1
			elif i != 0:
				numMonth = numMonth + 1
			#endif
			if (numMonth == int(self._currentDateSplit[1])) and (numYear == int(self._currentDateSplit[0])):
				isCurrentMonth.append(True)
			else:
				isCurrentMonth.append(False)
			#endif
			stdout = calendar.month(numYear, numMonth)
			rawcalendar = stdout.strip().strip().split("\n")
			lastDateMonth = rawcalendar[len(rawcalendar) - 1].strip().split(" ")
			lastDateMonth = lastDateMonth[len(lastDateMonth) - 1]
			calendarList.append(rawcalendar)
			appointmentDates = []
			for l in xrange(len(self._sortedList)):
				for j in xrange(1, (int(lastDateMonth) + 1)):
					checkDateString = str(numYear) + "-" + str(numMonth).rjust(2, "0") + "-" + str(j).rjust(2, "0")
					if self._sortedList[l][1] == checkDateString and (str(j) not in appointmentDates):
						appointmentDates.append(str(j))
					#endif
				#endfor
			#endfor
			appointmentDateList.append(appointmentDates)
		#endfor
		calendarTable = [[],[],[]]
		for i in xrange(3):
			for m in xrange(len(calendarList[i])):
				if m == 0:
					calendarTable[i].append(calendarList[i][m].strip())
				elif m == 1:
					calendarTable[i].append(calendarList[i][m].strip())
				elif len(calendarList[i][m].strip()) != 0:
					calendarTable[i].append([])
					for k in xrange(7):
						nextDate = calendarList[i][m][(k*3):((k*3) + 2)]
						if nextDate.strip(" ") in appointmentDateList[i]:
							if (isCurrentMonth[i] == True) and (str(self._currentDateSplit[2]) in appointmentDateList[i]) and (nextDate.strip(" ") == str(self._currentDateSplit[2])):
								calendarTable[i][m].append([('"' + nextDate + '"'),"Bold",_30minToApptColor_Lua,secondaryShadeColor_Lua])
							else:
								calendarTable[i][m].append([('"' + nextDate + '"'),"Normal",color0_Lua,defaultShadeColor_Lua])
							#endif
						else:
							if (isCurrentMonth[i] == True) and (str(self._currentDateSplit[2]) not in appointmentDateList[i]) and (nextDate.strip(" ") == str(self._currentDateSplit[2])):
								calendarTable[i][m].append([('"' + nextDate + '"'),"Bold",_240minToApptColor_Lua,secondaryShadeColor_Lua])
							else:
								calendarTable[i][m].append([('"' + nextDate + '"'),"Normal",defaultColor_Lua,defaultShadeColor_Lua])
							#endif
						#endif
					#endfor
				#endif
			#endfor
		#endfor
		return str(calendarTable).replace("[","{").replace("]","}").replace("'{","{").replace("}'","}").replace("'\"","'").replace("\"'","'")
	#enddef
	
#endclass
	
def dateInSeconds(startDate, startTime):
	timeStruct = time.strptime((startDate + " " + startTime), "%Y-%m-%d %H:%M:%S")
	return time.mktime(timeStruct)
#enddef

def parseTest():
	rawCalendar_ = open(icalendarPath, "rb") # open calendar for reading
	evoCalendar = rawCalendar_.read()
	rawCalendar_.close()
	####
	#print "importing"
	parsCalendar = Calendar.from_ical(evoCalendar)
	#print "walking"
	eventHolder = eventList()
	for component in parsCalendar.walk('VEVENT'):
		eventSummary = component['SUMMARY']
		compDTStart = component['DTSTART'].to_ical()
		if compDTStart.find("T") != (-1):
			[eventStartDate, eventStartTime] = str(vDatetime.from_ical(compDTStart)).split(" ")
		else:
			eventStartDate = compDTStart[0:4] + "-" + compDTStart[4:6] + "-" + compDTStart[6:8]
			eventStartTime = "00:00:00"
		#endif
		eventStartSeconds = dateInSeconds(eventStartDate, eventStartTime)
		compDTEnd = component['DTEND'].to_ical()
		if compDTEnd.find("T") != (-1):
			[eventEndDate, eventEndTime] = str(vDatetime.from_ical(compDTEnd)).split(" ")
		else:
			eventEndDate = compDTEnd[0:4] + "-" + compDTEnd[4:6] + "-" + compDTEnd[6:8]
			eventEndTime = "23:59:59"
		#endif
		try:
			compRRule = component['RRULE']
			eventRecurFreq = compRRule['FREQ']
		except Exception, e:
			compRRule = "N/A"
		#endtry
		weeklyType = 0
		dailyType = 0
		yearlyType = 0
		if compRRule == "N/A": # Single appointments
			eventHolder.appendSingleEvent([eventStartSeconds, eventStartDate, eventStartTime, eventEndDate, eventEndTime, eventSummary, compRRule])
		elif eventRecurFreq[0] == "WEEKLY": # Weekly reccuring appointments ## RRULE:FREQ=WEEKLY;UNTIL=20111209T040000;BYDAY=MO,WE,FR;COUNT=4
			try:
				eventRecurDays = compRRule['BYDAY']
			except Exception, e:
				print "Error Unknown WEEKLY Recurring Appointment Type: " + str(e)
				break
			#endtry
			try: # check for WEEKLY events defined by enddate
				eventRecurEndDate = compRRule['UNTIL'][0]
				weeklyType = 1
				eventRecurCount = "N/A"
			except Exception, e:
				pass
			#endtry
			try: # check for WEEKLY events defined by number of occurances
				eventRecurCount = compRRule['COUNT']
				weeklyType = 2
				eventRecurEndDate = "N/A"
			except Exception, e:
				pass
			#endtry
			try: # Find dates that do not have ocurrances (holidays, canceled classes, ect.)
				eventExceptions = component['EXDATE']
				exceptionsList = []
				try:
					for i in xrange(len(eventExceptions)):
						exceptionDateRaw = eventExceptions[i].to_ical()
						if exceptionDateRaw.find("T") != (-1):
							[exceptionDate, exceptionTime] = str(vDatetime.from_ical(exceptionDateRaw)).split(" ")
						else:
							exceptionDate = exceptionDateRaw[0:4] + "-" + exceptionDateRaw[4:6] + "-" + exceptionDateRaw[6:8]
							exceptionTime = eventStartTime
						#endif
						exceptionsList.append(dateInSeconds(exceptionDate, exceptionTime))
					#endfor
				except Exception, e: # only one exception (not put in list)
					exceptionDateRaw = eventExceptions.to_ical()
					if exceptionDateRaw.find("T") != (-1):
						[exceptionDate, exceptionTime] = str(vDatetime.from_ical(exceptionDateRaw)).split(" ")
					else:
						exceptionDate = exceptionDateRaw[0:4] + "-" + exceptionDateRaw[4:6] + "-" + exceptionDateRaw[6:8]
						exceptionTime = eventStartTime
					#endif
					exceptionsList.append(dateInSeconds(exceptionDate, exceptionTime))
				#endtry
			except Exception, e:
				#print str(e)
				pass
			#endtry
			if weeklyType != 0:
				eventHolder.appendReccuringEvent_Weekly(eventStartDate, eventStartTime, eventEndDate, eventEndTime, eventRecurDays, eventRecurEndDate, eventRecurCount, weeklyType, exceptionsList, eventSummary)
			#endif
		elif eventRecurFreq[0] == 'DAILY': # Daily reccruing appointments ## RRULE;X-EVOLUTION-ENDDATE=20120525T233000Z:FREQ=DAILY;COUNT=8;INTERVAL=2;
			try: # check for DAILY events that occur at a interval of every other day or greater
				eventRecurInterval = compRRule['INTERVAL']
				eventRecurCount = compRRule['COUNT']
				eventRecurEndDate = "N/A"
				dailyType = 1
			except Exception, e:
				#print str(e)
				pass
			#endtry
			if dailyType != 1:
				try: # check for DAILY
					eventRecurCount = compRRule['COUNT']
					dailyType = 2
					eventRecurEndDate = "N/A"
					eventRecurInterval = 1
				except Exception, e:
					print "Error Unknown DAILY Recurring Appointment Type: " + str(e) 
					#pass
				#endtry
			#endif
			try: # Find dates that do not have ocurrances (holidays, canceled classes, ect.)
				eventExceptions = component['EXDATE']
				exceptionsList = []
				try:
					for i in xrange(len(eventExceptions)):
						exceptionDateRaw = eventExceptions[i].to_ical()
						if exceptionDateRaw.find("T") != (-1):
							[exceptionDate, exceptionTime] = str(vDatetime.from_ical(exceptionDateRaw)).split(" ")
						else:
							exceptionDate = exceptionDateRaw[0:4] + "-" + exceptionDateRaw[4:6] + "-" + exceptionDateRaw[6:8]
							exceptionTime = eventStartTime
						#endif
						exceptionsList.append(dateInSeconds(exceptionDate, exceptionTime))
					#endfor
				except Exception, e: # only one exception (not put in list)
					exceptionDateRaw = eventExceptions.to_ical()
					if exceptionDateRaw.find("T") != (-1):
						[exceptionDate, exceptionTime] = str(vDatetime.from_ical(exceptionDateRaw)).split(" ")
					else:
						exceptionDate = exceptionDateRaw[0:4] + "-" + exceptionDateRaw[4:6] + "-" + exceptionDateRaw[6:8]
						exceptionTime = eventStartTime
					#endif
					exceptionsList.append(dateInSeconds(exceptionDate, exceptionTime))
				#endtry
			except Exception, e:
				#print str(e)
				pass
			#endtry
			if dailyType != 0:
				eventHolder.appendReccuringEvent_Daily(eventStartDate, eventStartTime, eventEndDate, eventEndTime, eventRecurEndDate, eventRecurCount, eventRecurInterval, dailyType, exceptionsList, eventSummary)
			#endif
			pass # for now
		elif eventRecurFreq[0] == 'MONTHLY':
			pass # this is the most complicated type and I don't use it, it can wait till later
		elif eventRecurFreq[0] == 'YEARLY': #RRULE;X-EVOLUTION-ENDDATE=20140511T000000Z:FREQ=YEARLY;COUNT=3;WKST=SA
			try: # check for YEARLY
				eventRecurCount = compRRule['COUNT']
				yearlyType = 1
			except Exception, e:
				#print str(e)
				print "Error Unknown YEARLY Recurring Appointment Type: " + str(e)
				#pass
			#endtry
			try: # Find dates that do not have ocurrances (holidays, canceled classes, ect.)
				eventExceptions = component['EXDATE']
				exceptionsList = []
				try:
					for i in xrange(len(eventExceptions)):
						exceptionDateRaw = eventExceptions[i].to_ical()
						if exceptionDateRaw.find("T") != (-1):
							[exceptionDate, exceptionTime] = str(vDatetime.from_ical(exceptionDateRaw)).split(" ")
						else:
							exceptionDate = exceptionDateRaw[0:4] + "-" + exceptionDateRaw[4:6] + "-" + exceptionDateRaw[6:8]
							exceptionTime = eventStartTime
						#endif
						exceptionsList.append(dateInSeconds(exceptionDate, exceptionTime))
					#endfor
				except Exception, e: # only one exception (not put in list)
					exceptionDateRaw = eventExceptions.to_ical()
					if exceptionDateRaw.find("T") != (-1):
						[exceptionDate, exceptionTime] = str(vDatetime.from_ical(exceptionDateRaw)).split(" ")
					else:
						exceptionDate = exceptionDateRaw[0:4] + "-" + exceptionDateRaw[4:6] + "-" + exceptionDateRaw[6:8]
						exceptionTime = eventStartTime
					#endif
					exceptionsList.append(dateInSeconds(exceptionDate, exceptionTime))
				#endtry
			except Exception, e:
				#print str(e)
				pass
			#endtry
			if yearlyType != 0:
				eventHolder.appendReccuringEvent_Yearly(eventStartDate, eventStartTime, eventEndDate, eventEndTime, eventRecurCount, yearlyType, exceptionsList, eventSummary)
			#endif
		else:
			print "Error Unknown Recurring Appointment Type: " + str(eventRecurFreq)
			#pass
		#endif
	#endfor
	eventHolder.sortList()
	sortedList = eventHolder.returnSortedList()
	finalOutput = u"return { "
	finalOutput += eventHolder.conkyApts_Lua() 
	finalOutput += " , " + eventHolder.conkyCalendar_threeMonth_Lua(0) + " }"
	print str(finalOutput).encode('utf-8')
#enddef

#########

parseTest()
