# -*- coding: utf-8 -*-
"""
Name: evolution_conky_8.py
Author: arclance
Date_Created:2011-11-08
Date_Updated:2012-07-13
	
Monitor DBus for notification messages when evolution recieves new-mail.
Once all messages have been read the displayed information for each inbox can be reset sending a dbus message.

Inspired by
evolution-newmail.py by Thomas Perl <thp@thpinfo.com>
http://thpinfo.com/2007/hacks/

Requires
	Mail Notification Plugin for Evolution - set to send over dbus
	un-check Notify New Messages For Inbox Only
	Make Inbox With A Unique Name For Each Email Address You Have And Use A Filtering Rule to Put Messages From That Address In That Folder

Changelog
v8.2 - 2102-07-13
	escape " in message subject for lua output

v8.1 - 2012-07-11
	add output format for reading by lua scripts

v8.0 - 2012-05-27
	cache mail data so the script can be reset without loosing data about unread messages.
	
v7.0.1 - 2012-05-27
	beep before writing new messages to disk (added "\a" to terminal output)

v7.0 - 2012-05-15
	process dbus output as dbus structs intead of strings (fix display of unicode characters)
	improve ignoring of messages about non-monitored folders

v6.1 - 2012-05-02
	keep track of total number of new messages until reset
	report Multiple for sender if more than one message is recieved at a time
	clean up code (remove commented out obsolete (replaced or did not work) code, removed obsolete (unused) code, remove unecessary imports)

v6.0 - 2012-04-28
	add dbus listener to catch commands from the user indicating that all messages in a mailbox have been read
	this will rest the display to indicate there are not new messages
		I am using conky buttons to send this message
"""

import dbus
import dbus.glib
import dbus.decorators
import gobject
try:
	import cPickle as pickle
	print "using cPickle"
except ImportError:
	import pickle
	print "using pickle"
#endtry

### User Configuration ###
logDirectory = ""
cacheDirectory = logDirectory + "/cache"
### List of folders to monitor for new mail
new_mail_folders = [
		'Inbox1',
		'Inbox2'
]
### End User Configuration ###

"""
reset a specific inbox with a command in this format
dbus-send --dest=org.freedesktop.DBus "/org/freedesktop/sample/object/name" "evolution.conky.reset" string:"InboxName"
"""
def reset_inbox(inboxName="1"):
	print "\nreset_inbox - " + inboxName
	if inboxName != "1":
		try:
			resetUpdate = "OutputFolder" + str(new_mail_folders.index(inboxName)) + ".resetMailUpdate()"
			exec resetUpdate
			formatOutput_Lua(True)
		except ValueError, e:
			print "Not A Monitored Folder - " + str(e)
		#endtry
	#endif

def receiving_newmail2( path, folder=[u"None"], numMessages=[0], msgID=[u"msgID"], msgSender=[u"msgSender"], msgSubject=[u"msgSubject"]):
	print "\nreceiving_newmail"
	#####
	folderParsed = folder[0]
	if folderParsed not in new_mail_folders: # don't waste time processing folders that are not being monitored
		print "Not A Monitored Folder - " + folderParsed
	else:
		print u"folderParsed = " + folderParsed
		#####
		numMessagesParsed = int(numMessages[0])
		print u"numMessagesParsed = " + str(numMessagesParsed)
		#####
		if msgSubject[0] != u"msgSubject": # dbus message contains a message subject (happens when only one email is recieved at a time)
			#####
			msgSenderParsed = msgSender[0].split(u"msg_sender:")[1]
			print u"msgSenderParsed = " + msgSenderParsed
			#####
			msgSubjectParsed = msgSubject[0].split(u"msg_subject:")[1]
			print u"msgSubjectParsed = " + msgSubjectParsed
			if msgSubjectParsed == "":
				msgSubjectParsed = u"No Subject"
			#endif
			#####
			try:
				newUpdate = "OutputFolder" + str(new_mail_folders.index(folderParsed)) + '.newMailUpdate(True, numMessagesParsed, msgSubjectParsed, msgSenderParsed)'
				exec newUpdate
				formatOutput_Lua()
			except ValueError, e:
				print "Not A Monitored Folder - " + str(e)
			#endtry
		else:
			try:
				newUpdate = "OutputFolder" + str(new_mail_folders.index(folderParsed)) + ".newMailUpdate(False, numMessagesParsed)"
				exec newUpdate
				formatOutput_Lua()
			except ValueError, e:
				print "Not A Monitored Folder - " + str(e)
			#endtry
		#endif
	#endif
#enddef

def formatOutput(isReset=False): # format and join output from multiple e-mail accounts
	terminalOutput = u""
	conkyOutput = u""
	for i in xrange(len(new_mail_folders)):
		infoUpdate = "newInfo" + str(i) + " = OutputFolder" + str(i) + ".createOutput()"
		exec infoUpdate
		newTerminalOutput = "terminalOutput = terminalOutput + str(newInfo" + str(i) + '[0])'
		exec newTerminalOutput
		terminalOutput = terminalOutput + "\n"
		newConkyOutput = "conkyOutput = conkyOutput + str(newInfo" + str(i) + '[1])'
		exec newConkyOutput
		conkyOutput = conkyOutput + "\n"
	#endfor
	conkyOutput = conkyOutput.strip()
	if isReset == False:
		print (terminalOutput.strip("\n") + "\a").encode('utf-8') # print to terminal for debugging and beep (\a) for audio warning without speakers
	else:
		print terminalOutput.strip("\n").encode('utf-8') # print to terminal for debugging
	#endif
	logFile = open(logDirectory + "/evolution_conky_Output.txt", "w")
	logFile.write(conkyOutput.encode('utf-8'))
	logFile.close()
#enddef

def formatOutput_Lua(isReset=False): # format and join output from multiple e-mail accounts
	terminalOutput = u""
	conkyOutput = u"return { "
	for i in xrange(len(new_mail_folders)):
		infoUpdate = "newInfo" + str(i) + " = OutputFolder" + str(i) + ".createOutput_Lua()"
		exec infoUpdate
		newTerminalOutput = "terminalOutput = terminalOutput + str(newInfo" + str(i) + '[0])'
		exec newTerminalOutput
		terminalOutput = terminalOutput + "\n"
		if i != 0:
			newConkyOutput = "conkyOutput = conkyOutput + ' , ' + str(newInfo" + str(i) + '[1])'
		else:
			newConkyOutput = "conkyOutput = conkyOutput + str(newInfo" + str(i) + '[1])'
		#endif
		exec newConkyOutput
	#endfor
	conkyOutput = conkyOutput + " }"
	if isReset == False:
		print (terminalOutput.strip("\n") + "\a").encode('utf-8') # print to terminal for debugging and beep (\a) for audio warning without speakers
	else:
		print terminalOutput.strip("\n").encode('utf-8') # print to terminal for debugging
	#endif
	logFile = open(logDirectory + "/evolution_conky_CB10_lua_log.txt", "w")
	logFile.write(conkyOutput.encode('utf-8'))
	logFile.close()
#enddef	

class make_output(object):
	
	def __init__(self, currentFolder, currentNewEmails=0, currentLatestMessage=u"", currentLatestSender=u""):
		self._currentFolder = u"" + currentFolder
		self._cacheFile = u"" + currentFolder + "_cache.txt"
		try:
			openCache = open((cacheDirectory + "/" + self._cacheFile), "r")
			[self._currentNewEmails, self._totalNewEmails, self._currentLatestMessage, self._currentLatestSender, self._printLatestMessage, self._newMessages] = pickle.load(openCache)
			openCache.close()
			print "Cache Loaded - " + self._currentFolder
		except Exception, e: # no cache file yet use default settings
			print "No Cache - Using Defaults"
			self._currentNewEmails = currentNewEmails # number of new messages recieved
			self._totalNewEmails = currentNewEmails # total number of new messages recieved since the last reset
			self._currentLatestMessage = currentLatestMessage
			self._currentLatestSender = currentLatestSender
			self._printLatestMessage = False # no latest message data yet do not try to print
			self._newMessages = False # no messages yet
		#endtry
		self._offset = "${voffset -3}"
		self._outputInfo = ("No New Messages In %s" % (self._currentFolder))
		self._outputInfoConky = ("%s${color0}No New Messages In ${color}%s" % (self._offset,self._currentFolder))
	#enddef
	
	def newMailUpdate(self, newMessageData=False, currentNewEmails=0, currentLatestMessage=u"", currentLatestSender=u""):
		if newMessageData == False:	# more than one email, don't get sender data from dbus messages
			self._currentNewEmails = currentNewEmails
			self._totalNewEmails = self._totalNewEmails + currentNewEmails
			self._newMessages = True
		else: # one new email, dbus message has sender data
			self._currentNewEmails = currentNewEmails
			self._totalNewEmails = self._totalNewEmails + currentNewEmails
			self._currentLatestMessage = u'' + currentLatestMessage.replace("$","$$").replace("#","\#")
			self._currentLatestSender = u'' + currentLatestSender
			self._newMessages = True
			self._printLatestMessage = True
		#endif
		### Update Cache When Output Changes ###
		openCache = open((cacheDirectory + "/" + self._cacheFile), "w")
		pickle.dump([self._currentNewEmails, self._totalNewEmails, self._currentLatestMessage, self._currentLatestSender, self._printLatestMessage, self._newMessages], openCache)
		openCache.close()
		### ###
	#enddef
	
	def resetMailUpdate(self):
		self._newMessages = False # Recieved Signal That All Messages In This Inbox Have Been Read
		self._printLatestMessage = False # no more messages so don't print the sender data
		self._currentLatestMessage = u""
		self._currentLatestSender = u""
		self._totalNewEmails = 0 # reset total number of new emails
		### Update Cache When Output Changes ###
		openCache = open((cacheDirectory + "/" + self._cacheFile), "w")
		pickle.dump([self._currentNewEmails, self._totalNewEmails, self._currentLatestMessage, self._currentLatestSender, self._printLatestMessage, self._newMessages], openCache)
		openCache.close()
		### ###
	#enddef
	
	def createOutput(self):
		if self._newMessages == False: # no new messages
			#print no messages output
			self._outputInfo = (u"No New Messages In %s" % (self._currentFolder))
			self._outputInfoConky = (u"%s${color0}No New Messages In ${color}%s" % (self._offset,self._currentFolder))
			#self._printLatestMessage = False # no more messages so don't print the sender data
		elif self._printLatestMessage == True: # have sender data so print it 
			self._outputInfo = (u'(%s) New Messages In %s From %s' % (self._totalNewEmails, self._currentFolder, self._currentLatestSender) + "\n" \
			+ '"%s"' % (self._currentLatestMessage))
			self._outputInfoConky = (u'%s${color0}(${color}%s${color0}) New Messages In ${color}%s ${color0}From ${color}%s' % (self._offset,self._totalNewEmails, self._currentFolder, self._currentLatestSender) + "\n" \
			+ '%s${color0}"${color}%s${color0}"' % (self._offset,self._currentLatestMessage))
		else: # don't have sender data so don't print it
			self._outputInfo = (u'(%s) New Messages In %s' % ( self._totalNewEmails, self._currentFolder))
			self._outputInfoConky = (u'%s${color0}(${color}%s${color0}) New Messages In ${color}%s ${color0}From ${color}Multiple' % (self._offset,self._totalNewEmails, self._currentFolder))	
		#endif
		return [self._outputInfo, self._outputInfoConky]					
	#enddef
	
	def createOutput_Lua(self):
		if self._newMessages == False: # no new messages
			#print no messages output
			self._outputInfo = (u"No New Messages In %s" % (self._currentFolder))
			self._outputInfoConky = (u'{0,"%s","",""}' % (self._currentFolder))
			#self._printLatestMessage = False # no more messages so don't print the sender data
		elif self._printLatestMessage == True: # have sender data so print it 
			self._outputInfo = (u'(%s) New Messages In %s From %s' % (self._totalNewEmails, self._currentFolder, self._currentLatestSender) + "\n" \
			+ '"%s"' % (self._currentLatestMessage.strip('"')))
			self._outputInfoConky = (u'{%s,"%s","%s","%s"}' % (self._totalNewEmails, self._currentFolder, self._currentLatestSender, self._currentLatestMessage.strip('"').replace('"','\"').replace('"','\\"')))
		else: # don't have sender data so don't print it
			self._outputInfo = (u'(%s) New Messages In %s' % (self._totalNewEmails, self._currentFolder))
			self._outputInfoConky = (u'{%s,"%s","Multiple",""}' % (self._totalNewEmails, self._currentFolder))	
		#endif
		return [self._outputInfo, self._outputInfoConky]					
	#enddef
#endclass				

### Initialize Folder Variables ###
for i in xrange(len(new_mail_folders)):
	make_Object = "OutputFolder" + str(i) + " = make_output('" + new_mail_folders[i] + "')"
	exec make_Object
#endfor
### End Initialize Folder Variables ###

bus = dbus.SessionBus()  # we are going to connect to the SessionBus

#### Setup Signals To Listen For ###
bus.add_signal_receiver( receiving_newmail2, signal_name = 'Newmail', dbus_interface = 'org.gnome.evolution.mail.dbus.Signal', path = '/org/gnome/evolution/mail/newmail')

bus.add_signal_receiver( reset_inbox, signal_name = 'reset', dbus_interface = 'evolution.conky', path = '/org/freedesktop/sample/object/name')

### Connect to SessionBus ###
loop = gobject.MainLoop()
loop.run()
